/*
* Licensed Materials - Property of IBM
* 5725-G92 (C) Copyright IBM Corp. 2011, 2013. All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or
* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/

WL.AuthHandler = function(someChallengeHandler) {
		
	function onLoginFormSubmit(reqURL, options) {
		sendLoginFormReply(reqURL, options);
	}

	/*  
	 * Sends the username and password to the server. The request that is sent  
	 * to the server is 'j_security_check'.  
	 */
	function sendLoginFormReply(reqURL, options) {
		
		function onLoginFormResponse(transport) {
			WL.Logger.debug("Response [login]");

			// protect against cases where the dummy request returns without a  
			// transport object  
			// for example, when the server is down.  
			if (transport && transport.getAllHeaders) {
				var headers = transport.getAllHeaders().split("\n");
				WL.CookieManager.handleResponseHeaders(headers);
			}
			if (Authenticator.isLoginFormResponse(transport)) {
				// never re-send the request is the response was a login form -  
				// because it may be an error message (wrong username/password)  
				WL.Client.__hideBusy();
				showLoginForm(transport);
			} else {
				if (typeof transport !== "undefined" && transport.responseJSON
						&& transport.responseJSON.notificationSubscriptionState) {
					if (!WL.Client.Push.__isDeviceSupportPush()) {
						WL.Logger.error("The current Android version "
								+ device.version
								+ " does not support push notifications.");
						sendRequests();
					} else {
						handleSubscriptions(transport.responseJSON.notificationSubscriptionState);
						document.addEventListener("readytosubscribe",
								function() {
									sendRequests();
								}, false);
					}
				} else {
					Authenticator.onHideLogin();
					WL.Client.__hideBusy();
					someChallengeHandler.submitSuccess();
				}
			}
		}

		WL.Client.sendLoginFormReply(reqURL, options, onLoginFormResponse);
	}

	function onLoginFormCancel(transport) {
		onAuthEnd();
		var transport = {};
		transport.responseJSON = {
			errorCode : WL.ErrorCode.LOGIN_FAILURE,
			errorMsg : 'login canceled'
		};
		var req = getRequest(transport);
		req.options.onFailure(transport);
	}

	function showLoginForm(transport) {
		// Cancel option should be available only if LOGIN_ON_DEMAND  
		var onFormCancel = null;
		if (WL.StaticAppProps.APP_LOGIN_TYPE === WL.AppLoginType.LOGIN_ON_DEMAND) {
			onFormCancel = function() {
				onLoginFormCancel(transport);
			};
		}
		Authenticator.onBeforeLogin(transport, WL.Client.getUsername(),
				onLoginFormSubmit, onFormCancel);

	}
	
	// Public methods  
	return {				
		onLoginFormCancel : onLoginFormCancel,
		onLoginFormSubmit : onLoginFormSubmit		
	};
};

///////////////////////
// Challenge Handler
///////////////////////
var formBasedChallengeHandler = WL.Client.createChallengeHandler("formRealm");

formBasedChallengeHandler.isInitialized = false;
formBasedChallengeHandler.AuthHandler = WL.AuthHandler(formBasedChallengeHandler);
formBasedChallengeHandler.isCustomResponse = function(transport) {
	if (typeof Authenticator == "undefined") {
		return false;
	}
	if (!this.isInitialized) {
		this.isInitialized = true;
		Authenticator.init();
	}
	var isLoginFormResponse = Authenticator.isLoginFormResponse(transport);
	if (isLoginFormResponse) {
		Authenticator.onBeforeLogin(transport, WL.Client.getUsername,
				this.AuthHandler.onLoginFormSubmit,
				this.AuthHandler.onLoginFormCancel);
		WL.Client.__hideBusy();
		Authenticator.onShowLogin();
	} else {
		Authenticator.onHideLogin();
	}
	return isLoginFormResponse;
};

// empty method because on onSubmit we \process the challenge\  
formBasedChallengeHandler.handleChallenge = function(obj) {

};


/* Workligth Authenticator skeleton; Uncomment and implement if your application requires authentication. */

var Authenticator = function () {

    // Private members should be placed here

    return {
        // Public members should be placed here
        init : function () {
              // Authenticator initialization code should be placed here
        },

        isLoginFormResponse : function (response) {
              // Returns true/false
              
              return false;
        },
        
        onBeforeLogin : function (response, username, onSubmit, onCancel) {
              
        },

       onShowLogin: function() {
              
       },
        
       onHideLogin: function(){
              
       }               
    }; 
}();
