/*
 *  Licensed Materials - Property of IBM
 *  5725-G92 (C) Copyright IBM Corp. 2011, 2013. All Rights Reserved.
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */
 
define(["dojo/_base/lang", 
        "dojox/encoding/base64",
	     "dojo/_base/xhr",
         "com/ibm/mobile/appcenter/InstallerNative"], function(lang, dce, xhr, installerNative){
    lang.getObject("com.ibm.mobile.appcenter.Connection", true);
    
    com.ibm.mobile.appcenter.Connection = {    		
    		
    		_stringToByteArray: function(pString) {    			
    			var b = [];
    			for(var i = 0; i < pString.length; ++i){
    				b.push(pString.charCodeAt(i));
    			}
    			return b; 			
    		},
    		
    		_getBase64Auth: function(pUsername, pPassword) {    			
    			var source = pUsername + ":" + pPassword;
    			var sourceBytes = this._stringToByteArray(source);    
    			var ret = "Basic " + dce.encode(sourceBytes);
    			return ret; 			
    		},
    		
    		getInstalledApplicationListURL: function(pServerURL, pDeviceId) {
    			return pServerURL + 
    				"service/device/" + 
    				pDeviceId + "/package";
    		},
    		
    		
    		getPingPath: function(pServerURL) {
    			return pServerURL + "service/auth";
    		},
    		
    		getApplicationVersionsListURL: function(pServerURL, pApplication) {
    			return pServerURL + 
  	          			"service/directory/" + 
  	          			pApplication.expkg;
    		},
    		
    		_getApplicationIconURLPrefix: function(pForApp, pDeviceId, pDpi) {
    			var defaultURL = null;
    			var iconRel = this._getRelPerDpi(pDpi);
    			var iconURLs = {};
            	for (var urlIdx = 0; urlIdx < pForApp.url.length; urlIdx++) {
            		
            		iconURLs[pForApp.url[urlIdx].rel] = "service/" + pForApp.url[urlIdx].value;
            		
            		// save the default in case we do not find perfect match
            		if (pForApp.url[urlIdx].rel == "icon") { 
            			defaultURL =  "service/" + 
            	          	     pForApp.url[urlIdx].value;
            		}
            		
            		// this is what we really need if exists
            		if (pForApp.url[urlIdx].rel == iconRel) { 
            			return  "service/" + 
        	          	     pForApp.url[urlIdx].value;
            		}            		            		
            	}
            	
            	// choose next best quality icon
            	var idx = this._iconDpi.indexOf(iconRel);
            	if(idx != -1){
            		for(var i=idx+1; i<this._iconDpi.length; i++){
            			var iconUrl = iconURLs[this._iconDpi[i]];
            			if(iconUrl){
            				return  iconUrl;
            			}
            		}
            	}
            	
            	// either icon rel link or null
            	return defaultURL;
    		},
    		
    		_iconDpi: ["icon_xhdpi", "icon_hdpi", "icon_mdpi", "icon_ldpi"],
    		
    		_getRelPerDpi: function(pDpi) {
    			if (pDpi == installerNative.DPI.DENSITY_LOW)
    				return "icon_ldpi";
    			if (pDpi == installerNative.DPI.DENSITY_MEDIUM)
    				return "icon_mdpi";
    			if (pDpi == installerNative.DPI.DENSITY_HIGH)
    				return "icon_hdpi";
    			if (pDpi == installerNative.DPI.DENSITY_XHIGH)
    				return "icon_xhdpi";
    			return "icon_hdpi"; // if not default icon is set, allow to look for other urls
    		},
    		
    		getApplicationIconURL: function(pServerURL, pForApp, pDeviceId, pDpi) {
    			var iconUrl =  this._getApplicationIconURLPrefix(pForApp, pDeviceId, pDpi);
    			if (iconUrl != null)
    			  return pServerURL + iconUrl; 
    			return null;
    		},
    		
    		getApplicationListURL: function(pServerURL, pDeviceId, pFilter, pRequestSuffix) {
            	var url = pServerURL + 
		  	          "service/device/" + 
		  	           pDeviceId + "/" + 
		  	          pRequestSuffix +
		  	          "?t=" + new Date().getTime() + 
		  	          pFilter;
            	return url;
    		},
    		
    		getFeedbackURL: function(pServerURL, pApp) {
    			return pServerURL + "service/application/feedback/" + pApp.os + "/" + pApp.pkg + "/" + pApp.version;
    		},
    		
    		xhrGet : function (pArgs) {	
		    	console.log("xhrGet calling |" + pArgs.url + "| " + pArgs.sync);
			    
		    	
		    	//if (device.name === "IRDRT") //motorola XOOM 
		    	//{		    		
		    		if (!pArgs.headers) {
		    			pArgs.headers = {};
		    		}
		    		pArgs.headers.Authorization = this._getBase64Auth(pArgs.user, pArgs.password);  
		    		pArgs.user = null;
		    		pArgs.password = null;
		    	//} 

		    	
		    	xhr.get(pArgs);
    		}, 
    		
    		xhrPost : function (pArgs) {	
		    	console.log("xhrPost calling |" + pArgs.url + "| " + pArgs.sync);
			    
		    	//if (device.name === "IRDRT") //motorola XOOM 
		    	//{		    		
		    		if (!pArgs.headers) {
		    			pArgs.headers = {};
		    		}
		    		pArgs.headers.Authorization = this._getBase64Auth(pArgs.user, pArgs.password);  
		    		pArgs.user = null;
		    		pArgs.passowrd = null;
		    	//} 

		    	
		    	xhr.post(pArgs);
    		}
    };

    com.ibm.mobile.appcenter.Connection.DEFAULT_TIMEOUT = 10000;		// 10 seconds;
    
    return com.ibm.mobile.appcenter.Connection;
});
