/*
 *  Licensed Materials - Property of IBM
 *  5725-G92 (C) Copyright IBM Corp. 2011, 2013. All Rights Reserved.
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

package com.ibm.appcenter.artifacts;

import org.json.me.JSONException;
import org.json.me.JSONObject;

public class CodModule 
{
	private String _filename;
	private String _url;
	private int _size;
	private byte[] _data;
	private int _sortkey;
	
	/**
	 * Creates an COD module from JSON.
	 * @param json The JSON object
	 * @throws JSONException
	 */
	public CodModule(JSONObject json)
	throws JSONException
	{
		_filename = json.getString("filename");
		_url = json.getString("link");
		_size = json.getInt("size");
		// data is loaded later when needed
		_data = null;
		calcSortKey();
	}
	
	/**
	 * Returns the filename.
	 */
	public String getFileName()
	{
		return _filename;
	}
	
	/**
	 * Returns the URL.
	 */
	public String getURL()
	{
		return _url;
	}
	
	/**
	 * Returns the size.
	 */
	public int getSize()
	{
		return _size;
	}
	
	/**
	 * Sets the data of the cod module.
	 * @param data
	 */
	public void setData(byte[] data)
	{
		_data = data;
	}
	
	/**
	 * Returns the data of the cod module.
	 */
	public byte[] getData()
	{
		return _data;
	}
	
	/**
	 * Return the sort key.
	 */
	public int getSortKey()
	{
		return _sortkey;
	}
	
	/**
	 * Calculate the sort key from the file name.
	 */
	public void calcSortKey()
	{
		// file name format:
		// XYZ.cod
		// XYZ-1.cod
		// XYZ-2.cod
		_sortkey = 0;
		int i = _filename.length() - 1;
		int posdot = -1;
		int posdsh = -1;
		boolean allDigits = true;
		while (i >= 0) {
			char c = _filename.charAt(i);
			if (c == '.') {
				if (posdot > 0) {
					allDigits = false;
					break;
				}
				posdot = i;
			} else if (c == '-') {
				posdsh = i;
				break;
			} else if (posdot > 0) {
				if (c < '0' || c > '9')
					allDigits = false;
			}
			i--;
		}
		if (allDigits && 0 <= posdsh && posdsh < posdot) {
			String s = _filename.substring(posdsh + 1, posdot);
			_sortkey = Integer.parseInt(s);
		}
	}
}
