/*
 *  Licensed Materials - Property of IBM
 *  5725-G92 (C) Copyright IBM Corp. 2011, 2013. All Rights Reserved.
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

package com.ibm.appcenter.connection;

import org.json.me.JSONObject;


/**
 * A POST REST call with JSON content type.
 */
public class JSONPostCall extends JSONRESTCall
{
	/**
	 * Creates a new JSON REST call.
	 * Notice that REST calls are executed in a different thread.
	 * Hence a continuation must be passed that executes the code when
	 * the REST call retrieves the answer.
	 * @param url       	  The URL, including all parameters.
	 * @param sendPayload     The payload on POST.
	 * @param timeout         The timeout in milliseconds.
	 * @param continuation    The continuation code when the call returns.
	 * @param continuationInEventThread  Whether the continuation should be
	 *                        executed in the event thread for GUI operations.
	 *                        Must be false if the continuation appends
	 *                        additional calls otherwise those calls never
	 *                        get executed.
	 * @param handleBusyIndicator Whether this call handles the busy indicator
	 *                        automatically. Should only be used when the
	 *                        continuation does not append any additional
	 *                        call.
	 */
	public JSONPostCall(String url,
				   	   JSONObject sendPayload,
				   	   int timeout,
				   	   JSONRESTCallContinuation continuation,
				   	   boolean continuationInEventThread,
				   	   boolean handleBusyIndicator)
	{
		super(url, POST, sendPayload, timeout,
		      continuation, continuationInEventThread, handleBusyIndicator);
	}
	
	/**
	 * Creates a new JSON REST call.
	 * Notice that REST calls are executed in a different thread.
	 * Hence a continuation must be passed that executes the code when
	 * the REST call retrieves the answer.
	 * @param url       	  The URL, including all parameters.
	 * @param sendPayload     The payload on POST.
	 * @param continuation    The continuation code when the call returns.
	 * @param continuationInEventThread  Whether the continuation should be
	 *                        executed in the event thread for GUI operations.
	 *                        Must be false if the continuation appends
	 *                        additional calls otherwise those calls never
	 *                        get executed.
	 * @param handleBusyIndicator Whether this call handles the busy indicator
	 *                        automatically. Should only be used when the
	 *                        continuation does not append any additional
	 *                        call.
	 */
	public JSONPostCall(String url,
				   	   JSONObject sendPayload,
				   	   JSONRESTCallContinuation continuation,
				   	   boolean continuationInEventThread,
				   	   boolean handleBusyIndicator)
	{
		this(url, sendPayload, ConnectionUtil.getTimeout(),
		      continuation, continuationInEventThread, handleBusyIndicator);
	}
}