/*
 *  Licensed Materials - Property of IBM
 *  5725-G92 (C) Copyright IBM Corp. 2011, 2013. All Rights Reserved.
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

package com.ibm.appcenter.connection;

import org.json.me.JSONObject;

public interface JSONRESTCallContinuation {

	/**
	 * Called when the REST call is successful.
	 * @param respone     The JSON object of the HTTP response.
	 *                    Can be null if the response is not JSON.
	 * @param contentType Content type of the HTTP response.
	 * @param bytes       Bytes of the content of the HTTP response.
	 */
	public void onSuccess(JSONObject response, String contentType, byte[] bytes);
	
	/**
	 * Called when the REST call fails.
	 * @param ex          Exception that may have occured. Can be null.
	 * @param status      HTTP Status code if the HTTP connection returned.
	 * @param respone     The JSON object of the HTTP response.
	 *                    Can be null if the response is not JSON.
	 * @param contentType Content type of the HTTP response.
	 * @param bytes       Bytes of the content of the HTTP response.
	 */
	public void onFailure(Exception ex, int status, JSONObject response, String contentType, byte[] bytes);
	
	/**
	 * Called when the REST call is canceled.
	 */
	public void onCancel();
}
