/*
 *  Licensed Materials - Property of IBM
 *  5725-G92 (C) Copyright IBM Corp. 2011, 2013. All Rights Reserved.
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

package com.ibm.appcenter.connection;

/**
 * The continuation code after a REST call.
 */
public interface RESTCallContinuation {

	/**
	 * Called when the REST call is successful.
	 * @param contentType Content type of the HTTP response.
	 * @param bytes       Bytes of the content of the HTTP response.
	 */
	public void onSuccess(String contentType, byte[] bytes);
	
	/**
	 * Called when the REST call fails.
	 * @param ex           Exception that may have occured. Can be null.
	 * @param status       HTTP Status code if the HTTP connection returned.
	 * @param contentType  Content type of the HTTP response.
	 * @param bytes        Bytes of the content of the HTTP response.
	 */
	public void onFailure(Exception ex, int status, String contentType, byte[] bytes);
	
	/**
	 * Called when the REST call is canceled.
	 */
	public void onCancel();
}
