/*
 *  Licensed Materials - Property of IBM
 *  5725-G92 (C) Copyright IBM Corp. 2011, 2013. All Rights Reserved.
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

package com.ibm.appcenter.i18n;

import net.rim.device.api.i18n.MessageFormat;
import net.rim.device.api.i18n.ResourceBundle;

/**
 * Utilities for internationalization.
 */
public class I18N
implements I18NResource
{
	private static ResourceBundle _res = ResourceBundle.getBundle(BUNDLE_ID, BUNDLE_NAME);
	
	/**
	 * Returns the resource bundle for this application.
	 */
	public static ResourceBundle getBundle()
	{
		return _res;
	}
	
	/**
	 * Returns a string for a given id.
	 * @param id The id, or key of the string in the bundle.
	 * @return The string.
	 */
	public static String getString(int id)
	{
		return getBundle().getString(id);
	}
	
	/**
	 * Returns a string for a given id and an argument.
	 * The resource string returned from the bundle must contain {0}.
	 * @param id The id
	 * @param arg The argument inserted in the string.
	 * @return The string.
	 */
	public static String getString(int id, Object arg)
	{
		String fmt = getBundle().getString(id);
		return MessageFormat.format(fmt, new Object[] { arg });
	}
	
	/**
	 * Returns a string for a given id and two arguments.
	 * The resource string returned from the bundle must contain {0} and {1}.
	 * @param id The id
	 * @param arg1 The argument inserted in the string.
	 * @param arg2 The argument inserted in the string.
	 * @return The string.
	 */
	public static String getString(int id, Object arg1, Object arg2)
	{
		String fmt = getBundle().getString(id);
		return MessageFormat.format(fmt, new Object[] { arg1, arg2 });
	}
}
