/*
 *  Licensed Materials - Property of IBM
 *  5725-G92 (C) Copyright IBM Corp. 2011, 2013. All Rights Reserved.
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

package com.ibm.appcenter.ui;

import org.json.me.JSONException;
import org.json.me.JSONObject;

import com.ibm.appcenter.artifacts.AppSummary;
import com.ibm.appcenter.artifacts.AppVersion;
import com.ibm.appcenter.artifacts.Feedback;
import com.ibm.appcenter.connection.FeedbackUtil;
import com.ibm.appcenter.connection.JSONRESTCallContinuation;
import com.ibm.appcenter.connection.RESTCallBatch;
import com.ibm.appcenter.i18n.I18N;
import com.ibm.appcenter.i18n.I18NResource;
import com.ibm.appcenter.system.BlackBerrySystem;
import com.ibm.appcenter.ui.model.FeedbackListModel;
import com.ibm.appcenter.ui.widget.EnhancedBitmapButtonField;
import com.ibm.appcenter.ui.widget.EnhancedLabelField;
import com.ibm.appcenter.ui.widget.LeadingTrailingManager;
import com.ibm.appcenter.ui.widget.RatingDisplayField;
import com.ibm.appcenter.ui.widget.SpinnerPopup;
import com.ibm.appcenter.util.BasicUtil;

import net.rim.device.api.system.Display;
import net.rim.device.api.ui.DrawStyle;
import net.rim.device.api.ui.Field;
import net.rim.device.api.ui.FieldChangeListener;
import net.rim.device.api.ui.Graphics;
import net.rim.device.api.ui.UiApplication;
import net.rim.device.api.ui.component.ButtonField;
import net.rim.device.api.ui.component.ListField;
import net.rim.device.api.ui.component.SeparatorField;
import net.rim.device.api.ui.container.HorizontalFieldManager;
import net.rim.device.api.ui.container.VerticalFieldManager;

public class AppFeedbackPane
extends AppPropertyPane
{
	private EnhancedLabelField _numRatingsField = null;
	private RatingDisplayField _ratingField = null;
	private ButtonField _submitRatingButton = null;
	private EnhancedBitmapButtonField _modeButton = null;
	private ListField _feedbackListField = null;
	private FeedbackListModel _feedbackModel = null;
	private Feedback[] _feedbacks = null;
	private int _totalNumberOfFeedbacks = 0;
	private int _pageSize = 20;
	private int _initialPageSize = 3;
	private RESTCallBatch _batch;
	private boolean _batchBusy = false;
	private boolean _panelIsForAllVersions;
	private boolean _showFeedbackForAllVersions = false;
	
	/**
	 * Creates the pane.
	 * @param title The title of the pane.
	 * @param parent The parent screen.
	 */
	public AppFeedbackPane(String title, AppPropertyScreen parent)
	{
		super(title, parent);
        _feedbacks = new Feedback[] {};
        _panelIsForAllVersions = parent.isForAllVersions();
	}
	
	/**
	 * Rebuilds the pane. This is called whenever
	 * the data about the application is refreshed.
	 */
	public void build()
	{
		log("build");
		
		if (_batch == null) {
			_batch = new RESTCallBatch(null);	
		}
		
		VerticalFieldManager content = getContent();
		
    	if (content == null) return;
    	
    	getParent().setSafeFocus();
    	log("hasFocus" + checkFocus());
    	content.deleteAll();
    	
        int margin = 3 * UIUtil.getNormalXAdvanceInPxl() / 2;
        
        // area for the stars
        int size = UIUtil.getLargeBoldFontHeightInPxl();
        if (size * 5 + 10 > Display.getWidth()) {
    		size = (Display.getWidth() - 10) / 5;
    	}
        _ratingField = new RatingDisplayField(size, Field.FIELD_VCENTER);
        _ratingField.setRating(getAverageRating());
        
        EnhancedLabelField label = 
        		new EnhancedLabelField(I18N.getString(AppFeedbackPaneAverageLabel), 
        							   Field.FIELD_VCENTER);
        _numRatingsField =
        		new EnhancedLabelField(I18N.getString(I18NResource.AppArtifactRatingNumberInfo, 
        											  BasicUtil.formatRatingCount(getNumberOfRatings())),
        							   Field.FIELD_VCENTER);
        
        HorizontalFieldManager hmgr = new HorizontalFieldManager(Field.FIELD_HCENTER);
        hmgr.add(label);
        hmgr.add(_ratingField);
        hmgr.add(_numRatingsField);
        hmgr.setMargin(margin, 0, margin, 0);
        if (getParent().isForAllVersions()) {
        	LeadingTrailingManager hmgr2 = new LeadingTrailingManager(Field.USE_ALL_WIDTH);
        	hmgr2.setMargin(margin, 0, margin, 0);
        	String key = (UIUtil.isLightOnDark() ? "light" : "dark");
        	_modeButton = new EnhancedBitmapButtonField("settingsscreen92" + key, size, Field.FIELD_TRAILING | Field.FIELD_VCENTER);
        	_modeButton.setChangeListener(new FieldChangeListener() {
				public void fieldChanged(Field field, int context) {
					selectFeedbackMode();
				}});
        	hmgr2.add(hmgr);
        	hmgr2.add(_modeButton);
        	content.add(hmgr2);
        } else {
        	content.add(hmgr);
        }
        
        
        // the submit feedback button
        if (canSubmitFeedback()) { 
        	_submitRatingButton = new ButtonField(I18N.getString(AppFeedbackPaneSubmitButtonTitle), Field.FIELD_HCENTER | ButtonField.CONSUME_CLICK);
    		_submitRatingButton.setMargin(margin, 0, margin, 0);
    		_submitRatingButton.setChangeListener(new FieldChangeListener() {

    			public void fieldChanged(Field field, int context) {
    				UIUtil.pushScreen(new FeedbackSubmissionScreen(getAppVersion(), getParent()));
    			}});
    	
        	content.add(_submitRatingButton);
        }
        
        SeparatorField line = new SeparatorField(SeparatorField.LINE_HORIZONTAL);
        content.add(line);
        
        _feedbackListField = new ListField() {
        	public boolean navigationClick(int status, int time) {
        		
        		if (getParent().isBusy()) return true;
        		
        		Field focus = this.getLeafFieldWithFocus();
        		
        		if (focus instanceof ListField) {
        			ListField listField= (ListField)focus;
        			int index = listField.getSelectedIndex();
        			if (_feedbackModel != null && index >= 0) {
        				Feedback feedback = (Feedback)_feedbackModel.get(this, index);
        				if (feedback == null) {
        					onMore();
        				} else {
        					UIUtil.pushScreen(new FeedbackDisplayScreen(feedback));
        				}
        			}
        		}
        		return true;
        	}
        	public boolean keyChar(char key, int status, int time)
        	{
        		if (key == ' ') {
        			getParent().getHeader().setInitialFocus();
        			return true;
        		}
        		return super.keyChar(key,  status, time);
        	}
        	public void paint(Graphics g) {
        		int oldColor = g.getColor();
        		try {
        			g.setColor(UIUtil.getNormalTextColor());
        			super.paint(g);
        		} finally {
        			g.setColor(oldColor);
        		}
        	}
        };
        _feedbackListField.setEmptyString(
        		I18N.getString(AppFeedbackPaneNoFeedbacksLabel), 
        		DrawStyle.HCENTER);
        _feedbackModel = new FeedbackListModel(_feedbackListField, _panelIsForAllVersions);
        _feedbackModel.setLogEnabled(getParent().isLogEnabled());
        _feedbackModel.setFeedbacks(_feedbacks, true);
        
        content.add(_feedbackListField);
        
        log("build finished " + content.getFieldCount() + ", next getting feedback");
        
        onRefresh(false);
	}
	
	/**
	 * Decides whether we can submit feedback.
	 */
	private boolean canSubmitFeedback()
	{
        String moduleGroup = getAppVersion().getModuleGroupName();
        String version = getAppVersion().getVersion();
        if (getParent().isForAllVersions()) {
        	return BlackBerrySystem.isModuleGroupInstalled(moduleGroup);
        } else {
        	return BlackBerrySystem.isModuleGroupInstalled(moduleGroup, version) == BlackBerrySystem.APPLICATION_INSTALLED;
        }
	}
	
	/**
	 * Enable or disable the interactive parts of the UI.
	 */
	public void setEnabled(boolean enable)
	{
		log("setEnabled " + enable);
		if (_submitRatingButton != null) {
			_submitRatingButton.setEnabled(enable);
		}
		if (_feedbackListField != null) {
			_feedbackListField.setEnabled(enable);
		}
		if (_modeButton != null) {
			_modeButton.setEnabled(enable);
		}
	}
	
	/**
	 * Selects the feedback mode: whether to show feedback
	 * for all versions or for the latest version.
	 */
	private void selectFeedbackMode()
	{
		String[] choices = { 
				I18N.getString(AppFeedbackPaneSelectFeedbackAllVersionsChoice),
				I18N.getString(AppFeedbackPaneSelectFeedbackLatestVersionChoice)
		};
		int[] values = { 0, 1 };
        
        SpinnerPopup popup =
        		new SpinnerPopup(
        				I18N.getString(AppFeedbackPaneSelectFeedbackShowVersion), 
        				choices, 
        				values) {
            protected void onSelect(int choice)
            {
            	boolean showForAllVersions = (choice == 0);
            	if (showForAllVersions != _showFeedbackForAllVersions) {
            		_showFeedbackForAllVersions = showForAllVersions;
            		onRefresh(false);
            	}
            }
        };
        int choice = (_showFeedbackForAllVersions ? 0 : 1);
        popup.setSelectedChoice(choice);
        UiApplication app = UiApplication.getUiApplication();
        app.pushScreen(popup);
	}
	
	/**
	 * Returns the average over all displayed ratings.
	 */
	private double getAverageRating()
	{
		if (_panelIsForAllVersions && _showFeedbackForAllVersions) {
			AppSummary app = (AppSummary)getAppVersion();
			return app.getRatingAllVersions();
		} else {
			AppVersion app = getAppVersion();
			return app.getRating();
		}
	}
	
	/**
	 * Returns the number of all displayed ratings.
	 */
	private long getNumberOfRatings()
	{
		if (_panelIsForAllVersions && _showFeedbackForAllVersions) {
			AppSummary app = (AppSummary)getAppVersion();
			return app.getNumberOfRatingsAllVersions();
		} else {
			AppVersion app = getAppVersion();
			return app.getNumberOfRatings();
		}
	}
	    
	/**
	 * Refresh the list of feedbacks.
	 */
	void onRefresh(boolean handleBusyInd)
	{
		if (_batchBusy) return;
		
		log("onRefresh");
		
		_ratingField.setRating(getAverageRating());
		_numRatingsField.setText(I18N.getString(AppArtifactRatingNumberInfo, 
												BasicUtil.formatRatingCount(getNumberOfRatings())));
		_batchBusy = true;
		int start = 0;
		int pageSize = (_feedbacks == null ? _initialPageSize : _feedbacks.length);
		if (pageSize < _initialPageSize) {
			pageSize = _initialPageSize;
		}
		String id = getAppVersion().getId();
		String version = getAppVersion().getVersion();
		if (_panelIsForAllVersions && _showFeedbackForAllVersions)
			version = null;
		FeedbackUtil.retrieveFeedbacks(
				id, 
				version,
				start, 
				pageSize, 
				"created", // ordering key, no I18N
				false, 
				_batch, 
				new JSONRESTCallContinuation() {

					public void onSuccess(JSONObject response,
							String contentType, byte[] bytes) {
						log("Retrieved feedback successfully!");
						try {
							int idx = _feedbackListField.getSelectedIndex();
							_feedbacks = Feedback.getFeedbacksFromJSON(response, "items");
							_totalNumberOfFeedbacks = Feedback.getTotalNumberOfFeedbacks(response);
							boolean hasMore = _feedbacks.length < _totalNumberOfFeedbacks;
							_feedbackModel.setFeedbacks(_feedbacks, hasMore);
							if (idx < _feedbacks.length) {
								_feedbackListField.setSelectedIndex(idx);
							} else {
								_feedbackListField.setSelectedIndex(_feedbacks.length-1);
							}
							_batchBusy = false;
						} catch (JSONException ex) {
							log("Exception when decoding feedback", ex);
							_batchBusy = false;
							_feedbacks = new Feedback[] {};
							_totalNumberOfFeedbacks = 0;
							_feedbackModel.setFeedbacks(_feedbacks, false);
							_feedbackListField.setSelectedIndex(0);
						}
					}

					public void onFailure(Exception ex, int status,
							JSONObject response, String contentType,
							byte[] bytes) {
						log("Failed to get feedbacks! Status=" + status);
						UIUtil.showConnectionStatus(status, ex);
						_batchBusy = false;
						_feedbacks = new Feedback[] {};
						_totalNumberOfFeedbacks = 0;
						_feedbackModel.setFeedbacks(_feedbacks, false);
						_feedbackListField.setSelectedIndex(0);
					}

					public void onCancel() {
						_batchBusy = false;
					}}, 
				true, 
				handleBusyInd);
	}
	
	/**
	 * Extend the list of feedbacks.
	 */
	void onMore()
	{
		if (_batchBusy) return;
		
		log("onMore");
		
		_batchBusy = true;
		int start = (_feedbacks == null ? 0 : _feedbacks.length);
		String id = getAppVersion().getId();
		String version = getAppVersion().getVersion();
		if (_panelIsForAllVersions && _showFeedbackForAllVersions)
			version = null;
		FeedbackUtil.retrieveFeedbacks(
				id, 
				version,
				start, 
				_pageSize, 
				"created",
				false, 
				_batch, 
				new JSONRESTCallContinuation() {

					public void onSuccess(JSONObject response,
							String contentType, byte[] bytes) {
						log("Retrieved more feedback successfully!");
						try {
							int idx = _feedbacks.length;
							Feedback[] newFeedbacks = Feedback.getFeedbacksFromJSON(response, "items");
							_totalNumberOfFeedbacks = Feedback.getTotalNumberOfFeedbacks(response);
							_feedbacks = merge(_feedbacks, newFeedbacks);
							boolean hasMore = _feedbacks.length < _totalNumberOfFeedbacks;
							_feedbackModel.setFeedbacks(_feedbacks, hasMore);
							if (idx < _feedbacks.length) {
								_feedbackListField.setSelectedIndex(idx);
							} else {
								_feedbackListField.setSelectedIndex(_feedbacks.length-1);
							}
							_batchBusy = false;
						} catch (JSONException ex) {
							log("Exception when decoding feedback", ex);
							_batchBusy = false;
							// don't change anything
						}
					}

					public void onFailure(Exception ex, int status,
							JSONObject response, String contentType,
							byte[] bytes) {
						log("Failed to get more feedbacks! Status=" + status);
						UIUtil.showConnectionStatus(status, ex);
						_batchBusy = false;
						// don't change anything
					}

					public void onCancel() {
						_batchBusy = false;
					}}, 
				true, 
				true);
	}
	
	/**
	 * Merge two arrays of feedbacks.
	 */
	private Feedback[] merge(Feedback[] a, Feedback[] b)
	{
		log("merge " + a + " " + b);
		if (a == null || a.length == 0) return b;
		if (b == null || b.length == 0) return a;
		
		Feedback[] result = null;
		int n = a.length;
		int m = b.length;
		if (a[n-1].getTimeStamp() == b[0].getTimeStamp()) {
			result = new Feedback[n + m - 1];
			System.arraycopy(a, 0, result, 0, n-1);
			System.arraycopy(b, 0, result, n-1, m);
		} else {
			result = new Feedback[n + m];
			System.arraycopy(a, 0, result, 0, n);
			System.arraycopy(b, 0, result, n, m);
		}
		
		return result;
	}
	
	/**
	 * Returns the pane name for logging.
	 */
	public String getPaneName()
	{
		return "AFeePane";
	}
}
