/*
 *  Licensed Materials - Property of IBM
 *  5725-G92 (C) Copyright IBM Corp. 2011, 2013. All Rights Reserved.
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

package com.ibm.appcenter.ui;

import com.ibm.appcenter.artifacts.AppSummary;
import com.ibm.appcenter.connection.RESTCallBatch;
import com.ibm.appcenter.i18n.I18N;
import com.ibm.appcenter.system.BlackBerrySystem;
import com.ibm.appcenter.ui.model.AppListModel;
import com.ibm.appcenter.ui.widget.HeaderLine;
import com.ibm.appcenter.ui.widget.WaitPopupManager;
import com.ibm.appcenter.util.BusyIndicator;

import net.rim.device.api.ui.DrawStyle;
import net.rim.device.api.ui.Field;
import net.rim.device.api.ui.Graphics;
import net.rim.device.api.ui.Manager;
import net.rim.device.api.ui.UiApplication;
import net.rim.device.api.ui.component.Dialog;
import net.rim.device.api.ui.component.ListField;
import net.rim.device.api.ui.component.StandardTitleBar;
import net.rim.device.api.ui.component.TitleBar;
import net.rim.device.api.ui.container.MainScreen;
import net.rim.device.api.ui.container.VerticalFieldManager;
import net.rim.device.api.util.Arrays;
import net.rim.device.api.util.Comparator;

/**
 * Base class for all screens that show a list of
 * applications.
 */
public abstract class AppListScreen
extends AppCenterScreen
{
	// search scopes
	private static final int NAME_SEARCH_SCOPE = 0;
	private static final int VERSION_SEARCH_SCOPE = 1;
	private static final int RATING_COUNT_SEARCH_SCOPE = 2;
	
	private static final String NAME_SEARCH_SCOPE_DISPLAY = I18N.getString(AppListScreenNameSearchScope);
	private static final String VERSION_SEARCH_SCOPE_DISPLAY = I18N.getString(AppListScreenVersionSearchScope);
	private static final String RATING_COUNT_SEARCH_SCOPE_DISPLAY = I18N.getString(AppListScreenRatingCountSearchScope);
	
	// sort scopes (even = asc., odd = desc.)
	private static final int NAME_ASC = 0;
	private static final int NAME_DESC = 1;
	private static final int RATING_VALUE_ASC = 2;
	private static final int RATING_VALUE_DESC = 3;
	private static final int UPDATED_ASC = 4;
	private static final int UPDATED_DESC = 5;
	private static final int POPULARITY_ASC = 6;
	private static final int POPULARITY_DESC = 7;
	
	private static final String NAME_ASC_SORT_DISPLAY = I18N.getString(AppListScreenNameAscOrdering);
	private static final String NAME_DESC_SORT_DISPLAY = I18N.getString(AppListScreenNameDescOrdering);
	private static final String NAME_SORT_DISPLAY = I18N.getString(AppListScreenNameOrdering);
	private static final String RATING_SORT_DISPLAY = I18N.getString(AppListScreenRatingValueOrdering);
	private static final String UPDATED_SORT_DISPLAY = I18N.getString(AppListScreenUpdatedOrdering);
	private static final String POPULARITY_SORT_DISPLAY = I18N.getString(AppListScreenPopularityOrdering);
	
	private HeaderLine _header;
	protected ListField _list;
	private AppListModel _model = null;
	private AppSummary[] _allApps = null;
	private String _searchString = null;
	private int _searchScope = NAME_SEARCH_SCOPE;
	private int _sortScope = NAME_ASC;
	
    /**
     * Creates the configuration screen.
     */
    public AppListScreen()
    {
    	super(MainScreen.NO_SYSTEM_MENU_ITEMS | Manager.NO_VERTICAL_SCROLL);

    	// Disable loging in certain debugging situations
    	// setLogEnabled(false);
    	
        TitleBar bar = getTitleBar();
        if (bar instanceof StandardTitleBar) {
        	StandardTitleBar sbar = (StandardTitleBar)bar;
        	sbar.addSignalIndicator();
        } else {
        	StandardTitleBar sbar = new StandardTitleBar();
        	sbar.addSignalIndicator();
        	setTitleBar(sbar);
        }
        
        boolean showSearchScope = UIUtil.isSearchScopeEnabled();
        
    	_header = new HeaderLine(true, true, showSearchScope, false, true) {
    		protected void onBack()
    		{
    			AppListScreen.this.onBack();
    		}
    		
    		protected void onRefresh()
    		{
    			IconCache.getInstance().clear();
    			BusyIndicator indicator = getBusyIndicator();
    			if (indicator instanceof WaitPopupManager) {
    				((WaitPopupManager)indicator).setBusyImmediate();
    			}
    			AppListScreen.this.onRefresh();
    		}
    		
    		protected void onSearch(String text, int searchScope, int sortScope)
    		{
    			AppListScreen.this.onSearch(text, searchScope, sortScope);
    		}
    		
    		protected String[] getSearchScopeDisplayNames()
    		{
    			return AppListScreen.this.getSearchScopeDisplayNames();
    		}
    		
    		protected int[] getSearchScopes()
    		{
    			return AppListScreen.this.getSearchScopes();
    		}
    		
    		protected String[] getSortScopeDisplayNames(boolean allowAscDesc)
    		{
    			return AppListScreen.this.getSortScopeDisplayNames(allowAscDesc);
    		}
    		
    		protected int[] getSortScopes(boolean allowAscDesc)
    		{
    			return AppListScreen.this.getSortScopes(allowAscDesc);
    		}
    	};
    	this.add(_header);
    	_header.setSearchScope(_searchScope);
    	_header.setSortScope(_sortScope);
    	
    	VerticalFieldManager mgr = new VerticalFieldManager(Manager.VERTICAL_SCROLL);
    	this.add(mgr);
    	
        _list = new ListField() {
        	public boolean navigationClick(int status, int time) {
        		
        		if (isBusy()) return true;
        		
        		Field focus = this.getLeafFieldWithFocus();
        		
        		if (focus instanceof ListField) {
        			ListField listField= (ListField)focus;
        			int index = listField.getSelectedIndex();
        			if (_model != null && index >= 0) {
        				AppSummary app = (AppSummary)_model.get(this, index);
        				app = convert(index, app);
        				UiApplication uiapp = UiApplication.getUiApplication();
        				uiapp.pushScreen(new AppPropertyScreen(app,
        													   app.getName(),
        	        										   app.getIcon(null, index)));
        			}
        		}
        		return true;
        	}
        	public boolean keyChar(char key, int status, int time)
        	{
        		if (key == ' ') {
        			_header.setInitialFocus();
        			return true;
        		}
        		return super.keyChar(key,  status, time);
        	}
        	public void paint(Graphics g) {
        		int oldColor = g.getColor();
        		try {
        			g.setColor(UIUtil.getNormalTextColor());
        			super.paint(g);
        		} finally {
        			g.setColor(oldColor);
        		}
        	}
        };
        _list.setEmptyString(I18N.getString(AppListScreenListMsgDuringRefresh), DrawStyle.HCENTER);
        _model = new AppListModel(_list);
        _model.setLogEnabled(isLogEnabled());
        setApps(getInitialApps());
        
        mgr.add(_list);
        
        addViewMenu();
        
        _header.setInitialFocus();
    }
    
    /**
     * Sets the title of the screen.
     */
    public void setTitle(String title)
    {
    	TitleBar bar = getTitleBar();
        if (bar instanceof StandardTitleBar) {
        	StandardTitleBar sbar = (StandardTitleBar)bar;       
            sbar.addTitle(title);
        }
    }
   	
	/**
	 * Sets whether the screen is busy.
	 * Normally this should disable interactions.
	 */
	public void setBusy(final boolean busy)
	{
		if (busy != isBusy()) {
			log("Busy changes from " + isBusy() + " to " + busy);
			super.setBusy(busy);
			UIUtil.invokeInEventThread(new Runnable() {
				public void run() {
					enableUI(!busy);
				}});
		}
	}
	
    /**
     * Enable all buttons.
     * @param enable Whether to enable or to disable.
     */
    private void enableUI(boolean enable)
    {
    	log("enableUI " + enable);
    	_header.setEnabled(enable);
    }
	
    /**
     * Sets the applications displayed here.
     * @param apps All apps, without filtering.
     */
    public void setApps(AppSummary[] apps)
    {
    	_allApps = apps;
    	_model.setApps(filterApps(apps, _searchString, _searchScope, _sortScope));
    }
    
    /**
     * Allows subclasses to add information to the app artifact.
     * @param index The index of this app in the list.
     * @param app The original app artifact.
     * @return    The resulting app artifact.
     */
    protected AppSummary convert(int index, AppSummary app)
    {
    	return app;
    }
    
    /**
     * Filters the apps according to search scope
     * and search string.
     * @param apps All apps.
     * @param searchString The search string.
     * @param searchScope The search scope.
     * @param sortScope The ordering scope.
     * @return The filtered apps.
     */
    protected AppSummary[] filterApps(AppSummary[] apps, String searchString, int searchScope, int sortScope)
    {
    	log("filterApps " + apps + " " + searchString + " " + searchScope + " " + sortScope);
    	if (apps == null || apps.length == 0)
    		return apps;
    	if (searchString == null || searchString.length() == 0)
    		return sortApps(apps, sortScope);
    	searchString = searchString.toUpperCase();
    	int count = 0;
    	String txt;
    	AppSummary[] filteredApps;
    	switch (searchScope) {
    		case NAME_SEARCH_SCOPE:
    			for (int i = 0; i < apps.length; i++) {
    				if (apps[i] == null) continue;
    				txt = apps[i].getName();
    				if (txt != null && txt.toUpperCase().indexOf(searchString) >= 0) {
    					count++;
    				}
    			}
    			filteredApps = new AppSummary[count];
    			count = 0;
    			for (int i = 0; i < apps.length; i++) {
    				if (apps[i] == null) continue;
    				txt = apps[i].getName();
    				if (txt != null && txt.toUpperCase().indexOf(searchString) >= 0) {
    					filteredApps[count++] = apps[i];
    				}
    			}
    			return sortApps(filteredApps, sortScope);
    			
    		case VERSION_SEARCH_SCOPE:
    			for (int i = 0; i < apps.length; i++) {
    				if (apps[i] == null) continue;
    				txt = apps[i].getVersion();
    				if (txt != null && txt.startsWith(searchString)) {
    					count++;
    				}
    			}
    			filteredApps = new AppSummary[count];
    			count = 0;
    			for (int i = 0; i < apps.length; i++) {
    				if (apps[i] == null) continue;
    				txt = apps[i].getVersion();
    				if (txt != null && txt.startsWith(searchString)) {
    					filteredApps[count++] = apps[i];
    				}
    			}
    			return sortApps(filteredApps, sortScope);
    			    			
    		case RATING_COUNT_SEARCH_SCOPE:
    			long searchRatingCount, c;
    			try {
    				searchRatingCount = Long.parseLong(searchString);
    			} catch (NumberFormatException e) {
    				searchRatingCount = Long.MAX_VALUE;
    			}
    			for (int i = 0; i < apps.length; i++) {
    				if (apps[i] == null) continue;
    				c = apps[i].getNumberOfRatings();
    				if (c >= searchRatingCount) {
    					count++;
    				}
    			}
    			filteredApps = new AppSummary[count];
    			count = 0;
    			for (int i = 0; i < apps.length; i++) {
    				if (apps[i] == null) continue;
    				c = apps[i].getNumberOfRatings();
    				if (c >= searchRatingCount) {
    					filteredApps[count++] = apps[i];
    				}
    			}
    			return sortApps(filteredApps, sortScope);
    	}
    	return sortApps(apps, sortScope);
    }
    
    /**
     * Sorts the applications.
     * @param apps The applications.
     * @param sortScope The ordering.
     * @return The sorted applications.
     */
    private AppSummary[] sortApps(AppSummary[] apps, int sortScope)
    {
    	if (apps == null || apps.length == 0)
    		return apps;
    	
    	Comparator c;
    	
    	switch (sortScope) {
    		case NAME_ASC:
    			c = new Comparator() {
					public int compare(Object o1, Object o2) {
						AppSummary a1 = (AppSummary)o1;
						AppSummary a2 = (AppSummary)o2;
						if (a1.getName() == null) return -1;
						if (a2.getName() == null) return 1;
						return a1.getName().toUpperCase().compareTo(a2.getName().toUpperCase());
					}};
    			break;
			case NAME_DESC:
    			c = new Comparator() {
					public int compare(Object o1, Object o2) {
						AppSummary a1 = (AppSummary)o1;
						AppSummary a2 = (AppSummary)o2;
						if (a1.getName() == null) return 1;
						if (a2.getName() == null) return -1;
						return -a1.getName().toUpperCase().compareTo(a2.getName().toUpperCase());
					}};
    			break;
			case RATING_VALUE_ASC:
    			c = new Comparator() {
					public int compare(Object o1, Object o2) {
						AppSummary a1 = (AppSummary)o1;
						AppSummary a2 = (AppSummary)o2;
						if (a1.getRating() < a2.getRating()) return -1;
						if (a1.getRating() > a2.getRating()) return 1;
						return 0;
					}};
    			break;
			case RATING_VALUE_DESC:
    			c = new Comparator() {
					public int compare(Object o1, Object o2) {
						AppSummary a1 = (AppSummary)o1;
						AppSummary a2 = (AppSummary)o2;
						if (a1.getRating() < a2.getRating()) return 1;
						if (a1.getRating() > a2.getRating()) return -1;
						return 0;
					}};
    			break;
    		case UPDATED_ASC:
      			c = new Comparator() {
    				public int compare(Object o1, Object o2) {
    					AppSummary a1 = (AppSummary)o1;
    					AppSummary a2 = (AppSummary)o2;
    					if (a1.getUpdated() < a2.getUpdated()) return -1;
    					if (a1.getUpdated() > a2.getUpdated()) return 1;
    					return 0;
    				}};
        		break;
    		case UPDATED_DESC:
    			c = new Comparator() {
					public int compare(Object o1, Object o2) {
						AppSummary a1 = (AppSummary)o1;
						AppSummary a2 = (AppSummary)o2;
						if (a1.getUpdated() < a2.getUpdated()) return 1;
						if (a1.getUpdated() > a2.getUpdated()) return -1;
						return 0;
					}};
    			break;
    		case POPULARITY_ASC:
      			c = new Comparator() {
    				public int compare(Object o1, Object o2) {
    					AppSummary a1 = (AppSummary)o1;
    					AppSummary a2 = (AppSummary)o2;
    					if (a1.getPopularity() < a2.getPopularity()) return -1;
    					if (a1.getPopularity() > a2.getPopularity()) return 1;
    					return 0;
    				}};
        		break;
    		case POPULARITY_DESC:
			default:
    			c = new Comparator() {
					public int compare(Object o1, Object o2) {
						AppSummary a1 = (AppSummary)o1;
						AppSummary a2 = (AppSummary)o2;
						if (a1.getPopularity() < a2.getPopularity()) return 1;
						if (a1.getPopularity() > a2.getPopularity()) return -1;
						return 0;
					}};
    			break;
    	}
    	
    	Arrays.sort(apps, c);
    	return apps;
    }
    
    /**
     * Called when the screen becomes visible or invisible.
     */
    protected void onUiEngineAttached(boolean attached)
    {
    	// super does the logging
    	super.onUiEngineAttached(attached);
    	
    	if (attached) {
    		onRefresh();
    	} else {
    		RESTCallBatch batch = _model.getActiveBatch();
    		if (batch != null) {
    			batch.stop();
    		}
    	}
    }
    
    protected void onExposed()
    {
    	// super does the logging
    	super.onExposed();
    	if (BlackBerrySystem.applicationsChanged) {
    		onRefresh();
    		BlackBerrySystem.applicationsChanged = false;
    	}
    }
    
    /**
     * Called when the back button was pressed.
     */
	protected void onBack()
	{
		getRESTCallBatch().stop();
		UiApplication.getUiApplication().popScreen(this);
	}
		
    /**
     * Called when there is need to refresh the model.
     */
    protected void onRefresh()
    {
		// only used for debugging
		Dialog.alert("Refresh button pressed");
		afterRefresh();
    }
    
    /**
     * Called when the refresh is finished.
     */
    protected void afterRefresh()
    {
    	_list.setEmptyString(I18N.getString(AppListScreenListEmptyMsg), DrawStyle.HCENTER);
    }
    
	/**
	 * Called when text was entered in the search.
	 * This is called for every character that was
	 * entered.
	 * @param text The entire text that was entered.
	 */
	protected void onSearch(String text, int searchScope, int sortScope)
	{
		_searchString = text;
		_searchScope = searchScope;
		_sortScope = sortScope;
		int mode = AppSummary.VERSION_INFO;
		if (UIUtil.isAdaptAppCatalogInfoLineToSorting()) {
			switch (sortScope) {
				default:
					mode = AppSummary.VERSION_INFO;
					break;
				case UPDATED_ASC:
				case UPDATED_DESC:
					mode = AppSummary.UPDATED_INFO;
					break;
				case POPULARITY_ASC:
				case POPULARITY_DESC:
					mode = AppSummary.POPULARITY_INFO;
					break;
			}
		}
		_model.setInfoLineMode(mode);
		_model.setApps(filterApps(_allApps, _searchString, _searchScope, _sortScope));
	}
	
	/**
	 * Returns the display names of the search scopes.
	 * The returned array must have the same size
	 * as the returned array of {@link #getSearchScopes()}. 
	 */
	protected String[] getSearchScopeDisplayNames()
	{
		return new String[] { 
			NAME_SEARCH_SCOPE_DISPLAY,
			VERSION_SEARCH_SCOPE_DISPLAY,
			RATING_COUNT_SEARCH_SCOPE_DISPLAY 
		};
	}
	
	/**
	 * Returns the search scopes.
	 * The returned array must have the same size
	 * as the returned array of {@link #getSearchScopeDisplayNames()}. 
	 */
	protected int[] getSearchScopes()
	{
		return new int[] { 
			NAME_SEARCH_SCOPE,
			VERSION_SEARCH_SCOPE,
			RATING_COUNT_SEARCH_SCOPE
		};
	}
	
	/**
	 * Returns the display names of the sort scopes.
	 * The returned array must have the same size
	 * as the returned array of {@link #getSortScopes()}. 
	 * @param allowAscDesc 
	 */
	protected String[] getSortScopeDisplayNames(boolean allowAscDesc)
	{
		if (allowAscDesc) {
			return new String[] { 
					NAME_SORT_DISPLAY,
					RATING_SORT_DISPLAY,
					UPDATED_SORT_DISPLAY,
					POPULARITY_SORT_DISPLAY,
			};
		} else {
			return new String[] { 
					NAME_ASC_SORT_DISPLAY,
					NAME_DESC_SORT_DISPLAY,
					RATING_SORT_DISPLAY,
					UPDATED_SORT_DISPLAY,
					POPULARITY_SORT_DISPLAY,
				};
		}
	}
	
	/**
	 * Returns the sort scopes.
	 * The returned array must have the same size
	 * as the returned array of {@link #getSortScopeDisplayNames()}. 
	 */
	protected int[] getSortScopes(boolean allowAscDesc)
	{
		if (allowAscDesc) {
			return new int[] { 
					NAME_ASC,
					RATING_VALUE_ASC,
					UPDATED_ASC,
					POPULARITY_ASC,
			};
		} else {
			return new int[] { 
					NAME_ASC,
					NAME_DESC,
					RATING_VALUE_DESC,
					UPDATED_DESC,
					POPULARITY_DESC,
				};
		}
	}
	
    /**
     * Called when the screen was edited and closed.
     */
    protected boolean onSave()
    {
    	// nothing to save
    	return true;
    }
    
    /**
     * Called when the screen was edited, to query whether to close
     * the screen.
     */
    protected boolean onSavePrompt()
    {
    	// the screen should always close
    	return true;
    }
    
    /**
     * Returns the initial apps.
     */
    private AppSummary[] getInitialApps()
    {
    	return null;
    }
}