/*
 *  Licensed Materials - Property of IBM
 *  5725-G92 (C) Copyright IBM Corp. 2011, 2013. All Rights Reserved.
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

package com.ibm.appcenter.ui;

import com.ibm.appcenter.artifacts.AppVersion;
import com.ibm.appcenter.i18n.I18NResource;
import com.ibm.appcenter.util.Log;

import net.rim.device.api.ui.Field;
import net.rim.device.api.ui.Manager;
import net.rim.device.api.ui.component.LabelField;
import net.rim.device.api.ui.component.pane.Pane;
import net.rim.device.api.ui.container.VerticalFieldManager;

/**
 * A pane of the application properties.
 */
public abstract class AppPropertyPane
extends Pane
implements I18NResource
{
	private VerticalFieldManager _content;
	private AppPropertyScreen _parent;
	
	/**
	 * Creates the pane.
	 * @param title The title of the pane.
	 * @param parent The parent screen.
	 */
	public AppPropertyPane(String title, AppPropertyScreen parent)
	{        
       	super(new LabelField(title, Field.FOCUSABLE | Field.FIELD_HCENTER),
       		  new VerticalFieldManager(Manager.USE_ALL_WIDTH | Manager.USE_ALL_HEIGHT | Manager.VERTICAL_SCROLL));
       	
       	log("Constructor " + title);
       	
       	_content = (VerticalFieldManager)getPane();
       	_content.setBackground(UIUtil.getNormalBackground());
       	_parent = parent;
	}
	
	/**
	 * Rebuilds the pane. This is called whenever
	 * the data about the application is refreshed.
	 */
	public abstract void build();

	/**
	 * Enable or disable the pane.
	 */
	public abstract void setEnabled(boolean enable);
	
	/**
	 * Returns the parent screen.
	 */
	public AppPropertyScreen getParent()
	{
		return _parent;
	}
	
	/**
	 * Returns the content of the pane.
	 */
	public VerticalFieldManager getContent()
	{
		return _content;
	}
	
	/**
	 * For debugging OS 6.0, check whether the panel has
	 * a focus.
	 */
	protected boolean checkFocus()
	{
		if (_content == null) return false;
		return checkFocus(_content);
	}
	
	private boolean checkFocus(Manager m)
	{
		if (m.isFocus()) {
			Log.log("Manager " + m.getClass().getName() + " has focus.");
			return true;
		}
		int n = m.getFieldCount();
		for (int i = 0; i < n; i++) {
			Field f = m.getField(i);
			if (f == null) continue;
			if (f instanceof Manager) {
				return checkFocus((Manager)f);
			} else if (f.isFocus()) {
				Log.log("Field " + f.getClass().getName() + " has focus.");
				return true;
			}
		}
		return false;
	}
	
	/**
	 * Returns the application to inspect.
	 */
	public AppVersion getAppVersion()
	{
		return getParent().getAppVersion();
	}
	
	/**
	 * Returns the pane name for logging.
	 */
	public abstract String getPaneName();
	
	private boolean _logEnabled = true;
	
	/**
	 * Enable or disable logging for this screen.
	 */
	public void setLogEnabled(boolean enable)
	{
		_logEnabled = enable;
	}
	
	/**
	 * Returns whether logging for this screen is enabled.
	 */
	public boolean isLogEnabled()
	{
		return _logEnabled;
	}
	
	/**
	 * Logs a throwable with a message.
	 * @param msg The message.
	 * @param th  The throwable.
	 */
	public void log(String msg, Throwable th)
	{
		if (isLogEnabled())
			Log.log(getPaneName() + " " + msg, th);
	}
	
	/**
	 * Logs a message.
	 * @param msg The message.
	 */
	public void log(String msg)
	{
		if (isLogEnabled())
			Log.log(getPaneName() + " " + msg);
	}
	
	/**
	 * Logs a message with arguments.
	 * @param msg The message.
	 * @param args The arguments.
	 *
	public void log(String msg, Object[] args)
	{
		if (isLogEnabled())
			Log.log(getPaneName() + " " + msg, args);
	}
	 */
}
