/*
 *  Licensed Materials - Property of IBM
 *  5725-G92 (C) Copyright IBM Corp. 2011, 2013. All Rights Reserved.
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

package com.ibm.appcenter.ui;

import org.json.me.JSONArray;
import org.json.me.JSONException;
import org.json.me.JSONObject;

import net.rim.device.api.system.Bitmap;
import net.rim.device.api.ui.Field;
import net.rim.device.api.ui.Graphics;
import net.rim.device.api.ui.Manager;
import net.rim.device.api.ui.UiApplication;
import net.rim.device.api.ui.component.BitmapField;
import net.rim.device.api.ui.component.LabelField;
import net.rim.device.api.ui.component.StandardTitleBar;
import net.rim.device.api.ui.component.TitleBar;
import net.rim.device.api.ui.component.pane.HorizontalScrollableController;
import net.rim.device.api.ui.component.pane.HorizontalScrollableTitleView;
import net.rim.device.api.ui.component.pane.PaneManagerModel;
import net.rim.device.api.ui.component.pane.PaneManagerView;
import net.rim.device.api.ui.component.pane.PaneView;
import net.rim.device.api.ui.container.HorizontalFieldManager;
import net.rim.device.api.ui.container.MainScreen;
import net.rim.device.api.ui.container.VerticalFieldManager;
import com.ibm.appcenter.artifacts.AppSummary;
import com.ibm.appcenter.artifacts.AppDetail;
import com.ibm.appcenter.artifacts.AppVersion;
import com.ibm.appcenter.connection.ApplicationUtil;
import com.ibm.appcenter.connection.ConnectionUtil;
import com.ibm.appcenter.connection.JSONRESTCall;
import com.ibm.appcenter.connection.JSONRESTCallContinuation;
import com.ibm.appcenter.connection.RESTCall;
import com.ibm.appcenter.connection.RESTCallBatch;
import com.ibm.appcenter.connection.RESTCallContinuation;
import com.ibm.appcenter.i18n.I18N;
import com.ibm.appcenter.system.BlackBerrySystem;
import com.ibm.appcenter.ui.widget.EnhancedLabelField;
import com.ibm.appcenter.ui.widget.HeaderLine;
import com.ibm.appcenter.ui.widget.WaitPopupManager;
import com.ibm.appcenter.util.BusyIndicator;
import com.ibm.appcenter.util.IconBitmapProvider;
import com.ibm.appcenter.util.Log;


public class AppPropertyScreen
extends AppCenterScreen
implements IconBitmapProvider
{
	private static Bitmap _fallbackAppIcon;
	private static int _iconSize = 0;
	
	private AppVersion _appVersion;
	private boolean _useLatestVersion;
	private String _refreshURL;
	private String _appLabel;
	private Bitmap _appIcon;
	
	private HeaderLine _header;
	
	private AppPropertyPane[] _panes = new AppPropertyPane[UIUtil.isVersionSelectionAllowed() ? 3 : 2];
	private boolean _feedbacksNeedRefresh = false;	
	
	private BitmapField _appIconField = new BitmapField(getFallbackAppIcon());
	private LabelField _appNameField = new EnhancedLabelField("", LabelField.ELLIPSIS);
	private LabelField _appVersionField = new EnhancedLabelField("", LabelField.ELLIPSIS);
		
	private VerticalFieldManager _mainMgr = new VerticalFieldManager();
	private HorizontalFieldManager _emptyTop = new HorizontalFieldManager();
	private HorizontalFieldManager _fullTop = new HorizontalFieldManager();
	private boolean _fullTopVisible;
		
    /**
     * Creates a new MyScreen object.
     * @param app The application artifact.
     * @param appLabel The label of the application.
     */
    public AppPropertyScreen(AppVersion app, String appLabel, Bitmap appIcon)
    {   
    	super(MainScreen.NO_SYSTEM_MENU_ITEMS | Manager.NO_VERTICAL_SCROLL);

    	log("Constructor " + app + " " + appLabel);
    	
    	// if the input is a version, we want info only of that version.
    	// if the input is an app summary, we want info about all versions,
    	// and in particular about the latest version.
    	_useLatestVersion = (app instanceof AppSummary);
    	_refreshURL = app.getURL();
    	if (app instanceof AppSummary) {
    		_refreshURL = ((AppSummary)app).getLatestVersionURL();
    	}
    	_appLabel = appLabel;
    	_appIcon = appIcon;
    	
        TitleBar bar = getTitleBar();
        if (bar instanceof StandardTitleBar) {
        	StandardTitleBar sbar = (StandardTitleBar)bar;
        	sbar.addSignalIndicator();
        	sbar.addTitle(I18N.getString(AppPropertyTitle));
        } else {
        	StandardTitleBar sbar = new StandardTitleBar();
        	sbar.addSignalIndicator();
        	sbar.addTitle(I18N.getString(AppPropertyTitle));
        	setTitleBar(sbar);
        }
        
    	_header = new HeaderLine(true, false, false, true, true) {
    		protected void onBack()
    		{
    			AppPropertyScreen.this.onBack();
    		}
    		
    		protected void onRefresh()
    		{
    			BusyIndicator indicator = getBusyIndicator();
    			if (indicator instanceof WaitPopupManager) {
    				((WaitPopupManager)indicator).setBusyImmediate();
    			}
    			AppPropertyScreen.this.onRefresh();
    		}
    		
    		protected void onSearch(String text, int searchScope, int sortScope)
    		{
    		}
    		
    		protected void onUpDown()
    		{
    			toggleTopPane();
    		}
    	};
    	this.add(_header);
    	
    	_mainMgr = new VerticalFieldManager();
    	this.add(_mainMgr);
    	
    
        _appVersion = app;
        
        _fullTop = new HorizontalFieldManager(Manager.USE_ALL_WIDTH);
        VerticalFieldManager vmgr = new VerticalFieldManager(Manager.USE_ALL_WIDTH);
        _fullTop.add(_appIconField);
        _fullTop.add(vmgr);
        _mainMgr.add(_fullTop);
        _fullTopVisible = true;
        
        // spacings
        int d = UIUtil.getNormalXAdvanceInPxl() / 2;
        _appIconField.setBitmap(_appIcon);
        _appIconField.setMargin(d, d, d, d);  // TRBL
        long style = LabelField.ELLIPSIS;
        if (_useLatestVersion) {
        	style |= Field.FIELD_VCENTER;
        }
        _appNameField = new EnhancedLabelField(_appLabel, style);
        _appNameField.setFont(UIUtil.getLargeBoldFont());         
            	
       	vmgr.add(_appNameField);
        
       	if (_useLatestVersion) {
       		int x = _appIcon != null ? _appIcon.getHeight() : IconCache.getDefaultIconDimension();
       		x += 2 * d - UIUtil.getLargeBoldFontHeightInPxl();
       		_appNameField.setMargin(x/2, d, x/2, 0); // TRBL
       	} else {
       		int x = _appIcon != null ? _appIcon.getHeight() : IconCache.getDefaultIconDimension();
       		x += 2 * d - UIUtil.getLargeBoldFontHeightInPxl() - UIUtil.getMicroFontHeightInPxl();
       		_appNameField.setMargin(x/2, d, 0, 0); // TRBL
       		_appVersionField.setMargin(0, d, x/2, 0);
       		_appVersionField.setFont(UIUtil.getMicroFont());
       		vmgr.add(_appVersionField);
       	}
        
        _mainMgr.add(getPaneView());
        _mainMgr.setBackground(UIUtil.getNormalBackground());
        
        addViewMenu();
    }
	
    /**
     * Returns the screen name for logging.
     */
    public String getScreenName()
    {
    	return "AppPropSCR";
    }
    
    /**
     * Returns the pane view that can contain multiple panes.
     */
    private PaneManagerView getPaneView()
    {
        // Instantiate the model for the pane manager and enable looping
        PaneManagerModel model = new PaneManagerModel();
        model.enableLooping(true);
        
        _panes[0] = new AppPropertyDetailPane(I18N.getString(AppPropertyScreenDetailsPaneTitle), this);
        _panes[1] = new AppFeedbackPane(I18N.getString(AppPropertyScreenFeedbacksPaneTitle), this);
        if (UIUtil.isVersionSelectionAllowed()) {
        	_panes[2] = new AppVersionPane(I18N.getString(AppPropertyScreenVersionsPaneTitle), this);
        }
        
        for (int i = 0; i < _panes.length; i++) {
        	if (_panes[i] != null) {
        		model.addPane(_panes[i]);
        		_panes[i].setLogEnabled(isLogEnabled());
        	}
        }
          
        // Choose which pane the model is displaying
        model.setCurrentlySelectedIndex(0);
        
        // Create the header and initialize the model and visual properties
        HorizontalScrollableTitleView header = new HorizontalScrollableTitleView(Field.FOCUSABLE) {
        	public boolean navigationClick(int status, int time) {
        		toggleTopPane();
        		return true;
        	}
        };
        header.enableArrows(false);
        header.setModel(model);
        header.setBackground(UIUtil.getScrollPaneHeaderBackground());
         
        // Create the PaneView object, which will display the panes and is
        // controlled by the model.
        PaneView paneView = new PaneView(Field.FOCUSABLE) {
        	public void paint(Graphics g) {
        		//XYRect rect = new XYRect(0,0,Display.getWidth(), Display.getHeight());
        		//UIUtil.getColorBackground(0x808080).draw(g, rect);
        		super.paint(g);
        	}
        };
        paneView.setModel(model);
        
        // Initialize the PaneManagerView
        PaneManagerView paneManagerView = new PaneManagerView(Field.FOCUSABLE, header, paneView);
        paneManagerView.setModel(model);
        model.setView(paneManagerView);
        
        // Initialize the Controller
        HorizontalScrollableController controller = new HorizontalScrollableController();
        controller.setModel(model);
        controller.setView(paneManagerView);
        model.setController(controller);
        paneManagerView.setController(controller);
        
        return paneManagerView;
    }
            
    /**
     * Returns the fallback icon.
     */
    private static Bitmap getFallbackAppIcon()
    {
    	if (_fallbackAppIcon == null) {
    		_iconSize = UIUtil.getBoldFontHeightInPxl() + 
    				2 * UIUtil.getMicroFontHeightInPxl();
    		_fallbackAppIcon = UIUtil.getScaledBitmap(
    				"blackberry.png",
    				_iconSize,
    				_iconSize);
    	}
    	return _fallbackAppIcon;
	}
	
    /**
     * Called when the back button was pressed.
     */
	protected void onBack()
	{
		getRESTCallBatch().stop();
		UiApplication.getUiApplication().popScreen(this);
	}
		
    /**
     * Called when there is need to refresh the model.
     */
    protected void onRefresh()
    {
		refresh();
    }
    
    /**
     * Called when the screen becomes visible or invisible.
     */
    protected void onUiEngineAttached(boolean attached)
    {
    	// super does the logging
    	super.onUiEngineAttached(attached);
    	
    	if (attached) {
    		refresh();
    	}
    }
    
    /**
     * Refresh the model.
     */
    private void refresh()
    {
    	if (_appVersion == null)
    		return;

    	ApplicationUtil.retrieveAppProperties(
    			_refreshURL,
    			getRESTCallBatch(),
    			new JSONRESTCallContinuation() {

    				public void onSuccess(JSONObject response,
    						String contentType, byte[] bytes) {
    					log("Retrieved app properties successfully");
    					try {
    						// optionally unpack a surrounding array
    						JSONArray array = JSONRESTCall.unwrapArray(response);
    						if (array != null && array.length() > 0) {
    							response = array.getJSONObject(0);
    						}
    						// now create the app detail artifact from the response
    						AppDetail app = new AppDetail(response);
    						BlackBerrySystem.updateFromCodeModuleManager(app, isForAllVersions());
    						_appVersion = app;
    						updateUI();
    					} catch (JSONException e) {
    						log("Error when parsing the JSON app properties!", e);
    					} finally {
    						afterRefresh();
    					}
    				}

    				public void onFailure(Exception ex, int status,
    						JSONObject response, String contentType,
    						byte[] bytes) {
    					log("Failed to retrieve app properties. Status=" + status);
    					afterRefresh();
    					UIUtil.showConnectionStatus(status, ex);
    				}

    				public void onCancel() {
    					afterRefresh();
    				}},
    			true,
    			true);
    }
    
    /**
     * Called in any case when the refresh is finished.
     */
    protected void afterRefresh()
    {
    }
    
    /**
     * Toggles the visibility of the top pane.
     */
    public void toggleTopPane()
    {
    	log("toggleTopPane, status=" + _fullTopVisible);
		if (_fullTopVisible) {
			_mainMgr.replace(_fullTop, _emptyTop);
			_fullTopVisible = false;
			_header.setUpDownButtonMode(false);
		} else {
			_mainMgr.replace(_emptyTop, _fullTop);
			_fullTopVisible = true;
			_header.setUpDownButtonMode(true);
		}
    }
    
	/**
	 * Sets whether the screen is busy.
	 * Normally this should disable interactions.
	 */
	public void setBusy(final boolean busy)
	{
		if (busy != isBusy()) {
			log("Busy changes from " + isBusy() + " to " + busy);
			super.setBusy(busy);
			UIUtil.invokeInEventThread(new Runnable() {
				public void run() {
					enableUI(!busy);
				}});
		}
	}
	
    /**
     * Enable all buttons.
     * @param enable Whether to enable or to disable.
     */
    private void enableUI(boolean enable)
    {
    	log("enableUI " + enable);
    	_header.setEnabled(enable);
    	for (int i = 0; i < _panes.length; i++) {
    		if (_panes[i] != null)
    			_panes[i].setEnabled(enable);
    	}
    }
    
    /**
     * Updates the UI.
     * @param app The app displayed in the UI.
     */
    void updateUI()
    {
    	log("updateUI");
    	// needed for BB 6 (Torch) since cannot build the pane
    	// if an old field has focus
    	setSafeFocus();
    	    	
    	if (!panesHaveFocus()) {
    		updateUIImpl();
    	} else {
    		// All this is stupid, but needed due to the focus bugs
    		// of BB 6 with touch screens
    		_updateUIRunnerCnt = 5;
    		_updateUIRunner = new Runnable() {
				public void run() {
					_updateUIRunnerCnt--;
					if (panesHaveFocus()) {
						Log.log("Still have focus, retry " + _updateUIRunnerCnt);
						setSafeFocus();
						if (_updateUIRunner != null && _updateUIRunnerCnt > 0) {
							UIUtil.invokeInEventThread(_updateUIRunner);
						} else {
							Log.log("Give up1");
						}
					} else {
						updateUIImpl();
					}
				}};
			UIUtil.invokeInEventThread(_updateUIRunner);
    	}
    }
    
    private Runnable _updateUIRunner;
    private int _updateUIRunnerCnt;
    
    void updateUIImpl()
    {
    	log("updateUIImpl");
    	_updateUIRunner = null;
    	
    	for (int i = 0; i < _panes.length; i++) {
    		if (_panes[i] != null)
    			_panes[i].build();
    	}
    	if (_appVersion instanceof AppSummary) {
    		AppSummary app = (AppSummary)_appVersion;
    		_appLabel = app.getName();
    		_appIcon = app.getIcon(this, 0);
        	_appNameField.setText(_appLabel);
        	_appIconField.setBitmap(_appIcon);
    	}
    	int key = (_useLatestVersion ? AppPropertyScreenHeaderLatestVersion
    								 : AppPropertyScreenHeaderVersion);
    	_appVersionField.setText(I18N.getString(key, _appVersion.getVersion()));
    	// updateInstallButtons();
        invalidate();
    }
    
    private boolean panesHaveFocus()
    {
    	boolean hasFocus = false;
    	for (int i = 0; i < _panes.length; i++) {
    		if (_panes[i] != null)
    			hasFocus |= _panes[i].checkFocus();
    	}
    	return hasFocus;
    }
    
    /**
	 * Returns the bitmap for a given url.
	 * @param url The url.
	 * @param listIndex Unused here, since this page has only 1 icon.
	 * @return The bitmap.
	 */
	public Bitmap getIcon(final String url, int listIndex)
	{
		// the bitmap should always be in the cache already
    	Bitmap bm = IconCache.getInstance().get(url);
    	if (bm != null) {
    		if (bm == IconCache.NO_BITMAP)
    			return getFallbackAppIcon();
    		return bm;
    	}
				
    	log("getIcon " + url + " " + listIndex);
    	// use a REST call to get the icon bitmap

    	RESTCall call = new RESTCall(
    			ConnectionUtil.getURI(url),
				ConnectionUtil.getUsername(),
				ConnectionUtil.getPassword(),
				RESTCall.GET,
				null,
				null,
		        ConnectionUtil.getTimeout(),
				new RESTCallContinuation() {

					public void onSuccess(String contentType, byte[] bytes) {
						log("Retrieved successfully icon " + url);
						Bitmap bm = null;
						try {
							// ensure _iconSize is initialized
							getFallbackAppIcon();
							// now handle the bitmap
							bm = Bitmap.createBitmapFromPNG(bytes, 0, bytes.length);
							bm = UIUtil.getScaledBitmap(bm, _iconSize, _iconSize);
							IconCache.getInstance().put(url, bm);
						} catch (Exception ex) {
							// ignore the exception. We cannot do much else,
							// since this is a bad time point for dialog etc.
							log("Could not decode bitmap " + url, ex);
							IconCache.getInstance().put(url, null);
							bm = getFallbackAppIcon();
						}
				    	// redraw the related list field. This will pickup the
				    	// bitmap from cache
						final Bitmap fbm = bm;
						UIUtil.invokeInEventThread(new Runnable(){
							public void run() {
								_appIconField.setBitmap(fbm);
							}}); 
					}

					public void onFailure(Exception ex, int status,
							String contentType, byte[] bytes) {
						log("Failed to retrieve icon " + url + ", status=" + status);
						// we simply ignore the failure, assuming the bitmap is null
						IconCache.getInstance().put(url, null);						
					}

					public void onCancel() {
						log("Cancelled to retrieve icon " + url);
					}},
				false,
				false);
    	
    	// Use our own batch
    	RESTCallBatch batch = new RESTCallBatch(null);
    	batch.append(call);
    	batch.start();
    	    	
    	return getFallbackAppIcon();
	}
	
	/**
	 * Needed for BB 6 (Torch) since cannot build the pane
   	 * if an old field has focus;
	 */
	void setSafeFocus()
	{
    	_header.setSafeFocus();
	}
	
    /**
     * Called when the screen was edited and closed.
     */
    protected boolean onSave()
    {
    	// nothing to save
    	return true;
    }
    
    /**
     * Called when the screen was edited, to query whether to close
     * the screen.
     */
    protected boolean onSavePrompt()
    {
    	// the screen should always close
    	return true;
    }
    
    /**
     * Called then the screen is exposed.
     */
    protected void onExposed()
    {
    	// super does the logging
    	super.onExposed();
    	if (_feedbacksNeedRefresh) {
    		((AppFeedbackPane)_panes[1]).onRefresh(false);
    		_feedbacksNeedRefresh = false;
    		BlackBerrySystem.applicationsChanged = true;
    	}
    }
    
    /**
     * Returns the application, or application version to inspect.
     */
    AppVersion getAppVersion()
    {
    	return _appVersion;
    }
    
    /**
     * Whether we show all versions, in particular the latest version.
     */
    boolean isForAllVersions()
    {
    	return _useLatestVersion;
    }
    
    /**
     * Returns the application label.
     */
    String getAppLabel()
    {
    	return _appLabel;
    }
    
    /**
     * Returns the icon of the application.
     */
    Bitmap getAppIcon()
    {
    	return _appIcon;
    }
    
    /**
     * Returns the header of the screen.
     */
    HeaderLine getHeader()
    {
    	return _header;
    }
    
    /**
     * Indicates that the feedback list has changed and needs refresh.
     */
    public void setFeedbacksNeedRefresh(boolean flag)
    {
    	_feedbacksNeedRefresh = flag;
    }
}
