/*
 *  Licensed Materials - Property of IBM
 *  5725-G92 (C) Copyright IBM Corp. 2011, 2013. All Rights Reserved.
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

package com.ibm.appcenter.ui;

import net.rim.device.api.system.Bitmap;
import net.rim.device.api.ui.DrawStyle;
import net.rim.device.api.ui.Field;
import net.rim.device.api.ui.Graphics;
import net.rim.device.api.ui.UiApplication;
import net.rim.device.api.ui.component.ListField;
import net.rim.device.api.ui.component.SeparatorField;
import net.rim.device.api.ui.container.VerticalFieldManager;

import com.ibm.appcenter.artifacts.AppDetail;
import com.ibm.appcenter.artifacts.AppVersion;
import com.ibm.appcenter.i18n.I18N;
import com.ibm.appcenter.ui.model.VersionListModel;

public class AppVersionPane 
extends AppPropertyPane
{
	private ListField _versionsListField = null;
	private VersionListModel _versionsModel = null;
	
	/**
	 * Creates the pane.
	 * @param title The title of the pane.
	 * @param parent The parent screen.
	 */
	public AppVersionPane(String title, AppPropertyScreen parent)
	{
		super(title, parent);
	}
	
	/**
	 * Rebuilds the pane. This is called whenever
	 * the data about the application is refreshed.
	 */
	public void build()
	{
		log("build");
		
		VerticalFieldManager content = getContent();
		
    	if (content == null) return;
    	
    	getParent().setSafeFocus();
    	log("hasFocus" + checkFocus());
    	content.deleteAll();
    	
        SeparatorField line = new SeparatorField(SeparatorField.LINE_HORIZONTAL);
        content.add(line);
        
        _versionsListField = new ListField() {
        	public boolean navigationClick(int status, int time) {
        		
        		if (getParent().isBusy()) return true;
        		
        		Field focus = this.getLeafFieldWithFocus();
        		
        		if (focus instanceof ListField) {
        			ListField listField= (ListField)focus;
        			int index = listField.getSelectedIndex();
        			if (_versionsModel != null && index >= 0) {
        				AppVersion version = (AppVersion)_versionsModel.get(this, index);
        				if (version != null) {
        					String label = getParent().getAppLabel();
        					Bitmap icon = getParent().getAppIcon();
        					UiApplication app = UiApplication.getUiApplication();
        					app.popScreen(getParent());
        					UIUtil.pushScreen(new AppPropertyScreen(version, label, icon));
        				}
        			}
        		}
        		return true;
        	}
        	public boolean keyChar(char key, int status, int time)
        	{
        		if (key == ' ') {
        			getParent().getHeader().setInitialFocus();
        			return true;
        		}
        		return super.keyChar(key,  status, time);
        	}
        	public void paint(Graphics g) {
        		int oldColor = g.getColor();
        		try {
        			g.setColor(UIUtil.getNormalTextColor());
        			super.paint(g);
        		} finally {
        			g.setColor(oldColor);
        		}
        	}
        };
        _versionsListField.setEmptyString(I18N.getString(AppVersionPaneNoVersionsLabel), DrawStyle.HCENTER);
        _versionsModel = new VersionListModel(_versionsListField);
        _versionsModel.setLogEnabled(getParent().isLogEnabled());
        if (getAppVersion() instanceof AppDetail) {
    		AppDetail app = (AppDetail)getAppVersion();
    		_versionsModel.setVersions(app.getVersions());
        }
        
        content.add(_versionsListField);
        
        log("build finished " + content.getFieldCount());
	}

	/**
	 * Enable or disable the interactive parts of the UI.
	 */
	public void setEnabled(boolean enable)
	{
		log("setEnabled " + enable);
		if (_versionsListField != null) {
			_versionsListField.setEnabled(enable);
		}
	}
	
	/**
	 * Returns the pane name for logging.
	 */
	public String getPaneName()
	{
		return "AVerPane";
	}
}
