/*
 *  Licensed Materials - Property of IBM
 *  5725-G92 (C) Copyright IBM Corp. 2011, 2013. All Rights Reserved.
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

package com.ibm.appcenter.ui;

import org.json.me.JSONException;
import org.json.me.JSONObject;

import com.ibm.appcenter.artifacts.AppSummary;
import com.ibm.appcenter.connection.ApplicationUtil;
import com.ibm.appcenter.connection.JSONRESTCallContinuation;
import com.ibm.appcenter.i18n.I18N;

public class CatalogScreen
extends AppListScreen
{	
    /**
     * Creates the configuration screen.
     */
    public CatalogScreen()
    {
    	super();
    	// super logs the constructor
    	
    	// Set the displayed title of the screen       
        setTitle(I18N.getString(CatalogScreenTitle));     
    }
       
    /**
     * Returns the screen name for logging.
     */
    public String getScreenName()
    {
    	return "CatSCR";
    }
    
    /**
     * Refresh the model.
     */
    protected void onRefresh()
    {
   		ApplicationUtil.retrieveCatalog(
   				getRESTCallBatch(),
    			new JSONRESTCallContinuation() {

    				public void onSuccess(JSONObject response,
    						String contentType, byte[] bytes) {
    					log("Retrieved app catalog successfully");
    					afterRefresh();
    					try {
    						AppSummary[] apps = AppSummary.getAppsFromJSON(response, "items");
    						setApps(apps);
    					} catch (JSONException e) {
    						log("Error when parsing the JSON app catalog!", e);
    					}	
    				}

    				public void onFailure(Exception ex, int status,
    						JSONObject response, String contentType,
    						byte[] bytes) {
    					log("Failed to retrieve app catalog! Status=" + status);
    					afterRefresh();
						UIUtil.showConnectionStatus(status, ex);		
    				}

    				public void onCancel() {
    					afterRefresh();
    				}},
    			true,
    			true);
    }
}
