/*
 *  Licensed Materials - Property of IBM
 *  5725-G92 (C) Copyright IBM Corp. 2011, 2013. All Rights Reserved.
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

package com.ibm.appcenter.ui;

import org.json.me.JSONException;
import org.json.me.JSONObject;

import net.rim.device.api.ui.Field;
import net.rim.device.api.ui.FieldChangeListener;
import net.rim.device.api.ui.Manager;
import net.rim.device.api.ui.UiApplication;
import net.rim.device.api.ui.component.ButtonField;
import net.rim.device.api.ui.component.LabelField;
import net.rim.device.api.ui.container.MainScreen;
import net.rim.device.api.ui.container.VerticalFieldManager;

import com.ibm.appcenter.connection.ConnectionUtil;
import com.ibm.appcenter.connection.DeviceUtil;
import com.ibm.appcenter.connection.JSONRESTCallContinuation;
import com.ibm.appcenter.i18n.I18N;
import com.ibm.appcenter.ui.widget.EnhancedLabelField;
import com.ibm.appcenter.util.Log;

public final class ConfigurationScreen 
extends AppCenterScreen
{	
	ConfigurationOptionsProvider _provider = new ConfigurationOptionsProvider();
	VerticalFieldManager _mgr;
	ButtonField _connectButton;
	
    /**
     * Creates the configuration screen.
     */
    public ConfigurationScreen()
    {   
    	super(MainScreen.NO_SYSTEM_MENU_ITEMS);
    	// super logs the constructor
    	
        // Set the displayed title of the screen       
        setTitle(I18N.getString(ConfigurationScreenTitle));
        
        _mgr = _provider.getVerticalFieldManager(false);
        
        FieldChangeListener listener = new FieldChangeListener() {
            public void fieldChanged(Field field, int context) {
                connect();
            }
        };
        _mgr.add(new LabelField(" "));
        _connectButton = new ButtonField(I18N.getString(ConfigurationScreenConnectButtonTitle), Field.FIELD_HCENTER | ButtonField.CONSUME_CLICK);
        _connectButton.setChangeListener(listener);
        _connectButton.setMargin(0, 0, UIUtil.getNormalFontHeightInPxl(), 0); // TRBL
        _mgr.add(_connectButton);
        
        int m = 3 * UIUtil.getNormalXAdvanceInPxl() / 2;
        LabelField label = new EnhancedLabelField("IBM Application Center");
        label.setFont(UIUtil.getLargeBoldFont());
        // setMargin params are TRBL
        label.setMargin(UIUtil.getMicroFontHeightInPxl(), m, 0, m);
        add(label);
        label = new EnhancedLabelField(I18N.getString(ConfigurationScreenIntroMsg));
        label.setMargin(0, m, 0, m);
        add(label);
        _mgr.setMargin(UIUtil.getNormalFontHeightInPxl(), m, 0, m);
        add(_mgr);
        this.setBackground(UIUtil.getNormalBackground());
        
        addViewMenu();
    }

    /**
     * Returns the screen name for logging.
     */
    public String getScreenName()
    {
    	return "ConfSCR";
    }
    
    /**
     * Connect to the server and registers the device.
     */
    private void connect()
    {
    	log("connect");
    	
        // tell the connections about it
    	ConfigurationData conf = _provider.getData();
    	ConnectionUtil.setConnection(conf);

        // save all data entered
        _provider.saveAndKeep();
        // avoid that we later ask for saving
        ConfigurationScreen.this.setDirty(false);

        log("Conf Screen displayed=" + this.isDisplayed() + " visible=" + this.isVisible());
        // try to register the device.
        // On success, we close this screen.
        // On failure, we stay on the screen.
        try {
        	DeviceUtil.registerDevice(
        		getRESTCallBatch(),
        		ConnectionUtil.getTimeout(),
        		new JSONRESTCallContinuation() {

    				public void onSuccess(JSONObject response,
    						String contentType,
    						byte[] bytes) {
    					log("Device registered successfully");
    					UiApplication app = UiApplication.getUiApplication();
    			        app.popScreen(ConfigurationScreen.this);
    				}

    				public void onFailure(Exception ex, 
    						int status,
    						JSONObject response, 
    						String contentType, 
    						byte[] bytes) {
    					
    					log("Failed to register device");
    					UIUtil.showConnectionStatus(status, ex);
    				}

    				public void onCancel() {
    				}
    			});
        } catch (JSONException ex) {
        	Log.log(ex);
        }
    }
  
	/**
	 * Sets whether the screen is busy.
	 * Normally this should disable interactions.
	 */
	public void setBusy(final boolean busy)
	{
		if (busy != isBusy()) {
			log("Busy changes from " + isBusy() + " to " + busy);
			super.setBusy(busy);
			UIUtil.invokeInEventThread(new Runnable() {
				public void run() {
					enableUI(!busy);
				}});
		}
	}
	
    /**
     * Enable all buttons.
     * @param enable Whether to enable or to disable.
     */
    private void enableUI(boolean enable)
    {
    	log("enableUI " + enable);
    	enableRecursive(_mgr, enable);
    	if (enable) {
    		_connectButton.setFocus();
    	}
    }
    
    private void enableRecursive(Field f, boolean enable)
    {
    	if (f instanceof Manager) {
    		Manager m = (Manager)f;
    		int n = m.getFieldCount();
    		for (int i = 0; i < n; i++) {
    			enableRecursive(m.getField(i), enable);
    		}
    	}
    	if (f != null) {
    		f.setEnabled(enable);
    	}
    }
    
    /**
     * Called when the user changed any field in the dialog.
     */
    protected boolean onSave()
    {
    	log("onSave");
    	_provider.save();
    	return super.onSave();
    }
        
}