/*
 *  Licensed Materials - Property of IBM
 *  5725-G92 (C) Copyright IBM Corp. 2011, 2013. All Rights Reserved.
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

package com.ibm.appcenter.ui;

import net.rim.device.api.system.Display;
import net.rim.device.api.ui.Field;
import net.rim.device.api.ui.FieldChangeListener;
import net.rim.device.api.ui.Manager;
import net.rim.device.api.ui.UiApplication;
import net.rim.device.api.ui.component.ButtonField;
import net.rim.device.api.ui.component.SeparatorField;
import net.rim.device.api.ui.component.StandardTitleBar;
import net.rim.device.api.ui.component.TitleBar;
import net.rim.device.api.ui.container.MainScreen;
import net.rim.device.api.ui.container.VerticalFieldManager;

import com.ibm.appcenter.artifacts.Feedback;
import com.ibm.appcenter.i18n.I18N;
import com.ibm.appcenter.ui.widget.EnhancedLabelField;
import com.ibm.appcenter.ui.widget.RatingDisplayField;

/**
 * This screen shows the details of a feedback.
 */
public class FeedbackDisplayScreen extends AppCenterScreen
{    	
	/**
	 * Constructor.
	 * @param app The app that can be rated.
	 */
	public FeedbackDisplayScreen(Feedback feedback)
	{
    	super(MainScreen.NO_SYSTEM_MENU_ITEMS | Manager.NO_VERTICAL_SCROLL);

    	log("Constructor " + feedback);
    	    	
    	// set the title bar
        TitleBar bar = getTitleBar();
        if (bar instanceof StandardTitleBar) {
        	StandardTitleBar sbar = (StandardTitleBar)bar;
        	sbar.addSignalIndicator();
        	sbar.addTitle(I18N.getString(FeedbackDisplayScreenTitle));
        } else {
        	StandardTitleBar sbar = new StandardTitleBar();
        	sbar.addSignalIndicator();
        	sbar.addTitle(I18N.getString(FeedbackDisplayScreenTitle));
        	setTitleBar(sbar);
        }
        
        // allocate a scrollable manager
    	VerticalFieldManager mgr = new VerticalFieldManager(Manager.VERTICAL_SCROLL);
    	add(mgr);
    	
    	// allocate the close button
        ButtonField closeButton = new ButtonField(I18N.getString(FeedbackDisplayScreenCloseButtonTitle), Field.FIELD_HCENTER | ButtonField.CONSUME_CLICK);
        
        // area for the stars
        int margin = 3 * UIUtil.getNormalXAdvanceInPxl() / 2;
        
        // add the author
        String msg = I18N.getString(FeedbackByAuthorMessage, feedback.getAuthor());
        EnhancedLabelField label = new EnhancedLabelField(msg);
        label.setFont(UIUtil.getLargeBoldFont());
        label.setMargin(margin, margin, 0, margin);
        mgr.add(label);
        
        // add the version
        msg = I18N.getString(FeedbackForVersionMessage, feedback.getVersion());
        label = new EnhancedLabelField(msg);
        label.setFont(UIUtil.getBoldFont());
        label.setMargin(margin, margin, 0, margin);
        mgr.add(label);
        
        // add the date
        msg = I18N.getString(FeedbackOnDateMessage, feedback.getTimeStampAsString());
        label = new EnhancedLabelField(msg);
        label.setFont(UIUtil.getBoldFont());
        label.setMargin(0, margin, 0, margin);
        mgr.add(label);
        
        // add the device
        msg = I18N.getString(FeedbackMakerModelMessage, feedback.getDeviceModel());
        label = new EnhancedLabelField(msg);
        label.setFont(UIUtil.getBoldFont());
        label.setMargin(0, margin, margin, margin);
        mgr.add(label);
        
        // add a separator
        SeparatorField line = new SeparatorField(SeparatorField.LINE_HORIZONTAL);
        mgr.add(line);
        
        // area for the stars
        int size = UIUtil.getLargeBoldFontHeightInPxl();
        if (size * 5 + 10 > Display.getWidth()) {
    		size = (Display.getWidth() - 10) / 5;
    	}
        RatingDisplayField ratingField = new RatingDisplayField(size, Field.FIELD_HCENTER);
        ratingField.setMargin(0, margin, 0, margin);
        ratingField.setRating(feedback.getRating());
        mgr.add(ratingField);
        
        // add a separator
        line = new SeparatorField(SeparatorField.LINE_HORIZONTAL);
        mgr.add(line);
        
        // add the comment
        label = new EnhancedLabelField(feedback.getComment());
        label.setFont(UIUtil.getNormalFont());
        label.setMargin(margin, margin, margin, margin);
        mgr.add(label);
        
        // add a separator
        line = new SeparatorField(SeparatorField.LINE_HORIZONTAL);
        mgr.add(line);
        
        // add the close button
        closeButton.setMargin(margin, 0, margin, 0); // TRBL
        mgr.add(closeButton);
               
        closeButton.setChangeListener(new FieldChangeListener() {
			public void fieldChanged(Field field, int context) {
				onBack();				
			}});
        
        addViewMenu();
	}
	
	/**
	 * Called when the back or cancel button is pressed.
	 */
	public void onBack()
	{
		setDirty(false);
		UiApplication.getUiApplication().popScreen(this);
	}
	
	/**
	 * Returns the screen name for logging.
	 */
	public String getScreenName()
	{
		return "FeeDisSCR"; // no I18N
	}
	
    /**
     * Called when the screen was edited and closed.
     */
    protected boolean onSave()
    {
    	// nothing to save
    	return true;
    }
    
    /**
     * Called when the screen was edited, to query whether to close
     * the screen.
     * @return true when we can close the screen, false otherwise.
     */
    protected boolean onSavePrompt()
    {
    	return true;
    }
}