/*
 *  Licensed Materials - Property of IBM
 *  5725-G92 (C) Copyright IBM Corp. 2011, 2013. All Rights Reserved.
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

package com.ibm.appcenter.ui;

import org.json.me.JSONException;
import org.json.me.JSONObject;

import net.rim.device.api.system.Bitmap;
import net.rim.device.api.ui.Field;
import net.rim.device.api.ui.FieldChangeListener;
import net.rim.device.api.ui.Graphics;
import net.rim.device.api.ui.Screen;
import net.rim.device.api.ui.UiApplication;
import net.rim.device.api.ui.XYRect;
import com.ibm.appcenter.connection.ConnectionUtil;
import com.ibm.appcenter.connection.DeviceUtil;
import com.ibm.appcenter.connection.JSONRESTCallContinuation;
import com.ibm.appcenter.i18n.I18N;
import com.ibm.appcenter.ui.widget.EnhancedLabelField;
import com.ibm.appcenter.ui.widget.EnhancedListStyleButtonField;
import com.ibm.appcenter.ui.widget.advancedui.ForegroundManager;
import com.ibm.appcenter.ui.widget.advancedui.ListStyleButtonField;
import com.ibm.appcenter.ui.widget.advancedui.ListStyleButtonSet;


/**
 * The entry screen that allows to branch into the subscreens.
 * Exiting this screen exits the entire application.
 */

public class StartScreen
extends AppCenterScreen
{
    private ForegroundManager _foreground;
    private ListStyleButtonField _settingsField;
    private ListStyleButtonField _myAppsField;
    private ListStyleButtonField _updatedAppsField;
    private ListStyleButtonField _catalogField;
    
    // only used when debugging
    // private ListStyleButtonField _installedAppsField;
    // private ListStyleButtonField _installedGroupsField;
    // private ListStyleButtonField _logField;
    
    private boolean _enabled = true;
    
    /**
     * Constructor.
     */
    public StartScreen() {
        
        super( NO_VERTICAL_SCROLL | USE_ALL_HEIGHT );
        
        String title = I18N.getString(StartScreenTitle);
        setTitle(title);
        setBackground(UIUtil.getNormalBackground());
        
        setLogEnabled(false);
        
        _foreground = new ForegroundManager() {
            protected void paintBackground( Graphics g )
            {
            	XYRect rect = new XYRect(0, getVerticalScroll(), getWidth(), getHeight());
            	UIUtil.getNormalBackground().draw(g, rect);
            }
        };

        ListStyleButtonSet buttonSet;
           
        EnhancedListStyleButtonField.setColors(
        		UIUtil.getButtonSetBackgroundColor(),
        		UIUtil.getButtonSetBorderColor(),
        		0x186DEF,
        		UIUtil.getNormalTextColor(),
        		UIUtil.getDisabledTextColor());
              
        EnhancedLabelField labelField =
        		new EnhancedLabelField(
        				I18N.getString(StartScreenWelcomeMsg));
        int m = 5 * UIUtil.getNormalFontHeightInPxl() / 4;
        int t = 2 * UIUtil.getNormalFontHeightInPxl() / 3;
        labelField.setMargin(t, m, t, m);
        
        // don't need the same msg twice
        if (title != null && !title.equals(I18N.getString(StartScreenWelcomeMsg))) {
        	_foreground.add(labelField);
    	}
        
        buttonSet = new ListStyleButtonSet();
        
        String key = UIUtil.isLightOnDark() ? "light" : "dark";
        
        int w = UIUtil.getNormalFontHeightInPxl();
        Bitmap norCaret = UIUtil.getScaledBitmap("arrowright" + key + ".png", w, w);
        Bitmap disCaret = UIUtil.getScaledBitmap("arrowright" + key + "_t50.png", w, w);
        
        w = 2 * UIUtil.getNormalFontHeightInPxl();
        Bitmap norIcon = UIUtil.getScaledBitmap("appcatalogscreen92" + key + ".png", w);
        Bitmap disIcon = UIUtil.getScaledBitmap("appcatalogscreen92" + key + "_t50.png", w);
        _catalogField =
        		new EnhancedListStyleButtonField(
        				norIcon,
        				disIcon,
        				I18N.getString(StartScreenAppCatalogButtonTitle), 
        				norCaret,
        				disCaret, 0);
        _catalogField.setChangeListener( new FieldChangeListener( ) {
            public void fieldChanged( Field field, int context ) {
            	if (isBusy()) return;
                pushScreen( new CatalogScreen() );
            }
        } );
        buttonSet.add( _catalogField );
        
        norIcon = UIUtil.getScaledBitmap("mymobilescreen92" + key + ".png", w);
        disIcon = UIUtil.getScaledBitmap("mymobilescreen92" + key + "_t50.png", w);
        _myAppsField = 
        		new EnhancedListStyleButtonField(
        				norIcon,
        				disIcon,
        				I18N.getString(StartScreenMyMobileButtonTitle), 
        				norCaret,
        				disCaret, 0);
        _myAppsField.setChangeListener( new FieldChangeListener( ) {
            public void fieldChanged( Field field, int context ) {
            	if (isBusy()) return;
                pushScreen( new MyAppsScreen() );
            }
        } );
        buttonSet.add( _myAppsField );
        
        norIcon = UIUtil.getScaledBitmap("updatesscreen92" + key + ".png", w);
        disIcon = UIUtil.getScaledBitmap("updatesscreen92" + key + "_t50.png", w);
        _updatedAppsField =
        		new EnhancedListStyleButtonField(
        				norIcon,
        				disIcon,
        				I18N.getString(StartScreenUpdatesButtonTitle),
        				norCaret,
        				disCaret, 0);
        _updatedAppsField.setChangeListener( new FieldChangeListener( ) {
            public void fieldChanged( Field field, int context ) {
            	if (isBusy()) return;
                pushScreen( new UpdatesScreen() );
            }
        } );
        buttonSet.add( _updatedAppsField );
        
        norIcon = UIUtil.getScaledBitmap("settingsscreen92" + key + ".png", w);
        disIcon = UIUtil.getScaledBitmap("settingsscreen92" + key + "_t50.png", w);
        _settingsField = 
        		new EnhancedListStyleButtonField(
        				norIcon,
        				disIcon,
        				I18N.getString(StartScreenSettingsButtonTitle),
        				norCaret,
        				disCaret, 0);
        _settingsField.setChangeListener( new FieldChangeListener( ) {
            public void fieldChanged( Field field, int context ) {
            	if (isBusy()) return;
                pushScreen( new ConfigurationScreen() );
            }
        } );
        buttonSet.add( _settingsField );
        buttonSet.setMargin(m, m, 3 * t, m); //TRBL
        
        _foreground.add( buttonSet );
        
        
        // Debug stuff, currently removed
        /*
        buttonSet = new ListStyleButtonSet();
        
        _installedAppsField = new ListStyleButtonField( "Installed Applications (debug)", norCaret );
        _installedAppsField.setChangeListener( new FieldChangeListener( ) {
            public void fieldChanged( Field field, int context ) {
            	if (isBusy()) return;
                pushScreen( new InstalledModulesScreen() );
            }
        } );
        buttonSet.add( _installedAppsField );
        
        _installedGroupsField = new ListStyleButtonField( "Installed Groups (debug)", norCaret );
        _installedGroupsField.setChangeListener( new FieldChangeListener( ) {
            public void fieldChanged( Field field, int context ) {
            	if (isBusy()) return;
                pushScreen( new InstalledGroupsScreen() );
            }
        } );
        buttonSet.add( _installedGroupsField );
                
        _logField = new ListStyleButtonField( "Log", norCaret );
        _logField.setChangeListener( new FieldChangeListener( ) {
            public void fieldChanged( Field field, int context ) {
                pushScreen( new LogScreen() );
            }
        } );
        buttonSet.add( _logField );
        
        //_foreground.add( buttonSet );
         */
        // End of debug stuff
            
        add( _foreground );
        
        // keep the UI disabled until the connection
        // is verified.
        enableUI(false);
        
        verifyConnection();
    }
    
    /**
     * Returns the screen name for logging.
     */
    public String getScreenName()
    {
    	return "StartSCR";
    }
    
    /**
     * Verify the connection.
     */
    private void verifyConnection()
    {
    	// Transfer the current connection, so that we can do
    	// HTTP REST calls right away
    	ConfigurationData conf = ConfigurationData.get();
    	ConnectionUtil.setConnection(conf);

		if (!UIUtil.isSavePasswordOnDevice() ||
			conf.getServer() == null || conf.getServer().length() == 0 ||
    		conf.getUser() == null || conf.getUser().length() == 0) {

			log("Start ConfigurationScreen since settings are empty");
    		enableUI(true);
    		UIUtil.invokeInEventThread(new Runnable() {
    			public void run() {
    				pushScreen(new ConfigurationScreen());
    			}
    		});
    	} else {
    		tryRegisterDevice();
    	}
    }
    
    /**
     * Try to register the device.
     * If no connection to the server, start the configuration screen.
     */
    private void tryRegisterDevice()
    {
    	log("tryRegisterDevice");
    	// for debugging
    	// ConnectionUtil.testURI();
    	
    	// now register
    	try {
    		DeviceUtil.registerDevice(
    			getRESTCallBatch(),
    			3000,
    			new JSONRESTCallContinuation() {

					public void onSuccess(JSONObject response,
							String contentType,
							byte[] bytes) {
						log("Sucessfully registered the device");
					}

					public void onFailure(Exception ex, 
							int status,
							JSONObject response, 
							String contentType, 
							byte[] bytes) {
						log("Start ConfigurationScreen since failed to authenticate. Status: " + status);
						UIUtil.pushScreen(new ConfigurationScreen());
					}

					public void onCancel() {
						log("Start ConfigurationScreen since cancel registration.");
						UIUtil.pushScreen(new ConfigurationScreen());
					}
				});
    	} catch (JSONException ex) {
    		log("Error when trying to call registerDevice", ex);
    	}
    }
    
	/**
	 * Sets whether the screen is busy.
	 * Normally this should disable interactions.
	 */
	public void setBusy(final boolean busy)
	{
		log(getScreenName() + " busy changes from " + isBusy() + " to " + busy);
		if (busy != isBusy()) {
			super.setBusy(busy);
			UIUtil.invokeInEventThread(new Runnable() {
				public void run() {
					enableUI(!busy);
				}});
		}
	}
	
    /**
     * Enable all buttons.
     * @param enable Whether to enable or to disable.
     */
    void enableUI(boolean enable)
    {
    	if (enable != _enabled) {
    		enableUIImpl(enable);
    		if (enable) {
    			_settingsField.setFocus();
    		}
    	}
    }
    
    void enableUIImpl(boolean enable)
    {
    	log("enableUI " + enable);
		_enabled = enable;
		_settingsField.setEnabled(enable);
		_myAppsField.setEnabled(enable && DeviceUtil.isRegistered());
		_catalogField.setEnabled(enable && DeviceUtil.isRegistered());
		_updatedAppsField.setEnabled(enable && DeviceUtil.isRegistered());    		
    }
    
    private void pushScreen( Screen toPush ) 
    {
        UiApplication.getUiApplication().pushScreen( toPush );
    }
            
    protected void onUiEngineAttached(boolean attached)
    {
    	// super does the logging
    	super.onUiEngineAttached(attached);
    }
    
    protected void onExposed()
    {
    	// super does the logging
    	super.onExposed();
    	if (DeviceUtil.isRegistered()) {
    		// don't call enableUI but enableUIImpl,
    		// otherwise the effect of changing isRegistered
    		// does not become visible
    		enableUIImpl(true);
    	}
    }
}

