/*
 *  Licensed Materials - Property of IBM
 *  5725-G92 (C) Copyright IBM Corp. 2011, 2013. All Rights Reserved.
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

package com.ibm.appcenter.ui;

import net.rim.device.api.ui.DrawStyle;

import org.json.me.JSONException;
import org.json.me.JSONObject;

import com.ibm.appcenter.artifacts.AppSummary;
import com.ibm.appcenter.connection.ApplicationUtil;
import com.ibm.appcenter.connection.JSONRESTCallContinuation;
import com.ibm.appcenter.i18n.I18N;

public class UpdatesScreen
extends AppListScreen
{	
    /**
     * Creates the configuration screen.
     */
    public UpdatesScreen()
    {
    	super();
    	// super logs the constructor
    	
    	// Set the displayed title of the screen       
        setTitle(I18N.getString(UpdatesScreenTitle));     
    }
      
    /**
     * Returns the screen name for logging.
     */
    public String getScreenName()
    {
    	return "UpdSCR";
    }
    
    /**
     * Refresh the model.
     */
    protected void onRefresh()
    {
   		ApplicationUtil.retrieveUpdates(
   				getRESTCallBatch(),
    			new JSONRESTCallContinuation() {

    				public void onSuccess(JSONObject response,
    						String contentType, byte[] bytes) {
    					log("Retrieved app updates successfully");
    					afterRefresh();
    					try {
    						AppSummary[] apps = AppSummary.getAppsFromJSON(response, "items");
    						setApps(apps);
    					} catch (JSONException e) {
    						log("Error when parsing the JSON app updates!", e);
    					}	
    				}

    				public void onFailure(Exception ex, int status,
    						JSONObject response, String contentType,
    						byte[] bytes) {
    					log("Failed to retrieve app updates! Status=" + status);
    					afterRefresh();
    					UIUtil.showConnectionStatus(status, ex);	
    				}

    				public void onCancel() {
    					afterRefresh();
    				}},
    			true,
    			true);
    }
    
    /**
     * Called when the refresh is finished.
     */
    protected void afterRefresh()
    {
    	_list.setEmptyString(I18N.getString(UpdatesScreenListEmptyMsg), DrawStyle.HCENTER);
    }
}
