/*
 *  Licensed Materials - Property of IBM
 *  5725-G92 (C) Copyright IBM Corp. 2011, 2013. All Rights Reserved.
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

package com.ibm.appcenter.ui.debug;

import java.util.Enumeration;

import com.ibm.appcenter.artifacts.AppDetail;
import com.ibm.appcenter.artifacts.AppSummary;
import com.ibm.appcenter.ui.AppListScreen;
import net.rim.device.api.system.Bitmap;
import net.rim.device.api.system.CodeModuleGroup;
import net.rim.device.api.system.CodeModuleGroupManager;

/**
 * This screen shows all of the really installed apps.
 */
public class InstalledGroupsScreen extends AppListScreen
{
	CodeModuleGroup[] _groups;
	
	public InstalledGroupsScreen()
	{	
		super();
		setTitle("Installed Apps");
		setApps(getInstalledApps());
	}
	
	private AppSummary[] getInstalledApps()
	{
		_groups = filter(CodeModuleGroupManager.loadAll());
		if (_groups == null) {
			return null;
		}
		int n = _groups.length;
		AppSummary[] apps = new AppSummary[n];
		for (int i = 0; i < n; i++) {
			String name = _groups[i].getName();
			Enumeration e = _groups[i].getModules();
			boolean first = true;
			while (e.hasMoreElements()) {
				String s = (String)e.nextElement();
				if (first) name += " (";
				else name += ", ";
				name += s;
				first = false;
			}
			if (!first)
				name += ")";
			String id = _groups[i].getFriendlyName();
			String version = _groups[i].getVersion();
			apps[i] = new AppSummary(name, id, version, null, 0);
		}
		
		return apps;
	}
	
    /**
     * Returns the screen name for logging.
     */
    public String getScreenName()
    {
    	return "InstGrpSCR";
    }
    
	private CodeModuleGroup[] filter(CodeModuleGroup[] groups)
	{
		return groups;
	}
		
    /**
     * Converts the AppSummary into an AppDetail by looking into the code manager.
     * @param index The index of this app in the list.
     * @param app The original app artifact.
     * @return    The resulting app artifact.
     */
    protected AppSummary convert(int index, AppSummary app)
    {
    	if (_groups == null)
    		return app;
    	String name = app.getName();
    	String id = app.getId();
    	String version = app.getVersion();
    	Bitmap icon = app.getIcon(null, index);
    	String description = _groups[index].getDescription();
    	String vendor = _groups[index].getVendor();
    	int size = 0;
    	long downloaded = 0;
		long installed = 0;
    	app = new AppDetail(name, id, version, icon, description, vendor, -1L, downloaded, installed, size);
    	return app;
    }
		
    /**
     * Refresh the model.
     */
    protected void onRefresh()
    {
    	setApps(getInstalledApps());
    	afterRefresh();
    }
}
