/*
 *  Licensed Materials - Property of IBM
 *  5725-G92 (C) Copyright IBM Corp. 2011, 2013. All Rights Reserved.
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

package com.ibm.appcenter.ui.debug;

import java.util.Enumeration;

import com.ibm.appcenter.artifacts.AppDetail;
import com.ibm.appcenter.artifacts.AppSummary;
import com.ibm.appcenter.ui.AppListScreen;

import net.rim.device.api.system.ApplicationDescriptor;
import net.rim.device.api.system.Bitmap;
import net.rim.device.api.system.CodeModuleGroup;
import net.rim.device.api.system.CodeModuleGroupManager;
import net.rim.device.api.system.CodeModuleManager;
import net.rim.device.api.system.EncodedImage;

/**
 * This screen shows all of the really installed modules.
 */
public class InstalledModulesScreen extends AppListScreen
{
	int[] _handles;
	
	public InstalledModulesScreen()
	{	
		super();
		setTitle("Installed Apps");
		setApps(getInstalledApps());
	}
	
	private AppSummary[] getInstalledApps()
	{
		_handles = filter(CodeModuleManager.getModuleHandles(false));
		if (_handles == null)
			return null;
		int n = _handles.length;
		AppSummary[] apps = new AppSummary[n];
		for (int i = 0; i < n; i++) {
			String name = CodeModuleManager.getModuleName(_handles[i]);
			if (CodeModuleManager.isLibrary(_handles[i])) {
				name += " #LIB#";
			}
			else if (CodeModuleManager.isMidlet(_handles[i])) {
				name += " #MDL#";
			}
			else {
				name += " #APP#";
			}
			name += getGroup(CodeModuleManager.getModuleName(_handles[i]));
			String version = CodeModuleManager.getModuleVersion(_handles[i]);
			apps[i] = new AppSummary(name, name, version, searchIcon(_handles[i]), 0);
		}
		
		return apps;
	}
	
    /**
     * Returns the screen name for logging.
     */
    public String getScreenName()
    {
    	return "InstModSCR";
    }
    
	int[] filter(int[] handles)
	{		
		return handles;
	}
	
	CodeModuleGroup[] _groups = null;
	
	/**
	 * Returns the group of a module.
	 */
	String getGroup(String name)
	{
		if (_groups == null)
			_groups = CodeModuleGroupManager.loadAll();
		for (int i = 0; i < _groups.length; i++) {
			Enumeration e = _groups[i].getModules();
			while (e.hasMoreElements()) {
				String s = (String)e.nextElement();
				if (name.equals(s)) {
					return " " + _groups[i].getName() + " " + _groups[i].getFriendlyName();
				}
			}
		}
		return "";
	}
	
    /**
     * Converts the AppSummary into an AppDetail by looking into the code manager.
     * @param index The index of this app in the list.
     * @param app The original app artifact.
     * @return    The resulting app artifact.
     */
    protected AppSummary convert(int index, AppSummary app)
    {
    	if (_handles == null)
    		return app;
    	String name = app.getName();
    	String version = app.getVersion();
    	Bitmap icon = app.getIcon(null, index);
    	String description = CodeModuleManager.getModuleDescription(_handles[index]);
    	String vendor = CodeModuleManager.getModuleVendor(_handles[index]); 
    	int size = CodeModuleManager.getModuleCodeSize(_handles[index]);
    	long downloaded = CodeModuleManager.getModuleDownloadTimestamp(_handles[index]);
		long installed = CodeModuleManager.getModuleTimestamp(_handles[index]);
    	app = new AppDetail(name, name, version, icon, description, vendor, -1L, downloaded, installed, size);
    	return app;
    }
	
	private Bitmap searchIcon(int handle)
	{
		ApplicationDescriptor[] descrs = CodeModuleManager.getApplicationDescriptors(handle);
		if (descrs == null) return null;
		for (int i = 0; i < descrs.length; i++) {
			EncodedImage img = descrs[i].getEncodedIcon();
			if (img==null) continue;
			Bitmap bm = img.getBitmap();
			if (bm != null) return bm;
		}
		return null;
	}
	
    /**
     * Refresh the model.
     */
    protected void onRefresh()
    {
    	setApps(getInstalledApps());
    	afterRefresh();
    }
}
