/*
 *  Licensed Materials - Property of IBM
 *  5725-G92 (C) Copyright IBM Corp. 2011, 2013. All Rights Reserved.
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

package com.ibm.appcenter.ui.widget;

import com.ibm.appcenter.i18n.I18N;
import com.ibm.appcenter.i18n.I18NResource;
import com.ibm.appcenter.ui.UIUtil;
import com.ibm.appcenter.util.Log;

import net.rim.device.api.ui.Field;
import net.rim.device.api.ui.FieldChangeListener;
import net.rim.device.api.ui.Screen;
import net.rim.device.api.ui.Touchscreen;
import net.rim.device.api.ui.component.ButtonField;
import net.rim.device.api.ui.component.LabelField;
import net.rim.device.api.ui.component.SeparatorField;
import net.rim.device.api.ui.component.TextSpinBoxField;
import net.rim.device.api.ui.container.HorizontalFieldManager;
import net.rim.device.api.ui.container.PopupScreen;
import net.rim.device.api.ui.container.SpinBoxFieldManager;
import net.rim.device.api.ui.container.VerticalFieldManager;

/**
 * A popup screen containing one spinner.
 */  
public class DoubleSpinnerPopup extends PopupScreen       
{  
    private TextSpinBoxField _spinBoxField1;
    private TextSpinBoxField _spinBoxField2;
    private boolean _isSet;  
    private int[][] _values;
      
    /** 
     * Creates a new CustomSpinnerPopup object 
     */  
    public DoubleSpinnerPopup(String title, String[] choices1, String[] choices2, int[][] values)  
    {
        super(new VerticalFieldManager(), Screen.DEFAULT_CLOSE);          
        _spinBoxField1 = new TextSpinBoxField(choices1);  
        _spinBoxField2 = new TextSpinBoxField(choices2); 
        
        SpinBoxFieldManager spinBoxManager = new SpinBoxFieldManager();

        // add the spin boxes
        spinBoxManager.add(_spinBoxField1);
        spinBoxManager.add(_spinBoxField2);

        // adjust properties
        spinBoxManager.setVisibleRows(3);
        spinBoxManager.setRowHeight(2 * UIUtil.getNormalFontHeightInPxl());
        
        add(new LabelField(title));  
        add(new SeparatorField());  
        HorizontalFieldManager hfm = new HorizontalFieldManager(Field.FIELD_HCENTER);  
        hfm.add(spinBoxManager);  
        add(hfm);
        if (Touchscreen.isSupported()) {
        	// add ok button, because the spinbox does not close on touch tab.
        	// For non-touchscreens, it is not needed since the spinbox closes
        	// on navigationClick, but for touchscreens, users might prefer the
        	// touch tab over the navigationClick.
            ButtonField closeButton = new ButtonField(I18N.getString(I18NResource.FeedbackDisplayScreenCloseButtonTitle), Field.FIELD_HCENTER | ButtonField.CONSUME_CLICK);
            int margin = UIUtil.getNormalXAdvanceInPxl();
            closeButton.setMargin(margin, 0, margin, 0); // TRBL
            add(closeButton);
                   
            closeButton.setChangeListener(new FieldChangeListener() {
    			public void fieldChanged(Field field, int context) {
    				invokeAction(ACTION_INVOKE);				
    			}});
        }
        _values = values;
    }  
            
    /** 
     * Retrieves the currently selected choice 
     * @return The currently selected choice 
     */  
    public int getChoice()  
    {
    	int index1 = _spinBoxField1.getSelectedIndex();
    	if (index1 < 0 || index1 >= _values.length)
    		return -1;
    	int index2 = _spinBoxField2.getSelectedIndex();
    	if (index2 < 0 || index2 >= _values[index1].length)
    		return -1;
    	return _values[index1][index2];
    }  
    
    /**
     * Selects the input choice.
     * @param choice
     */
    public void setSelectedChoice(int choice)
    {
    	for (int i = 0; i < _values.length; i++) {
    		for (int j = 0; j < _values[i].length; j++) {
    			if (_values[i][j] == choice) {
    				_spinBoxField1.setSelectedIndex(i);
    				_spinBoxField2.setSelectedIndex(j);
    				return;
    			}
    		}
    	}
    }
      
    /** 
     * Indicates whether the TextSpinBoxField has changed from 
     * its initial state. 
     * @return True if the selected choice has been modified, otherwise false 
     */  
    public boolean isSet()  
    {  
        return _isSet;  
    }          
          
    /** 
     * @see Screen#invokeAction(int) 
     */  
    protected boolean invokeAction(int action)  
    {  
        if (action == ACTION_INVOKE) {  
            if (!_isSet) {                      
                _isSet = true;  
            }
            onSelect(getChoice());
            close();  
            return true;  
        }  
        return false;  
    }      
    
    /**
     * Called when the user selected a choice.
     * @param choice The choice.
     */
    protected void onSelect(int choice)
    {
    }
      
    /** 
     * @see Screen#close() 
     */  
    public void close()  
    {  
    	Log.log("SprinnerPopup Close");
        if(!isSet()) {                  
            _spinBoxField1.setSelectedIndex(0);
            _spinBoxField2.setSelectedIndex(0);
        }  
        super.close();  
    }    
}