/*
 *  Licensed Materials - Property of IBM
 *  5725-G92 (C) Copyright IBM Corp. 2011, 2013. All Rights Reserved.
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

package com.ibm.appcenter.ui.widget;

import com.ibm.appcenter.ui.UIUtil;
import com.ibm.appcenter.ui.widget.advancedui.BitmapButtonField;

import net.rim.device.api.system.Bitmap;
import net.rim.device.api.ui.Graphics;
import net.rim.device.api.ui.TouchEvent;
import net.rim.device.api.ui.XYRect;

/**
 * An enhanced bitmap button.
 */
public class EnhancedBitmapButtonField extends BitmapButtonField
{
	private int _bitmapHeight = UIUtil.getToolbarIconSizeInPxl();
	private String _bitmapName;
	
	/**
	 * Constructor.
	 * @param bitmapName The base name of the bitmap without the .png suffix.
	 * @param size The width or height of the square bitmap.
	 * @param style The style.
	 */
	public EnhancedBitmapButtonField(String bitmapName, int size, long style)
	{
		super(getBitmap(bitmapName + ".png", size), style);
		_bitmapHeight = size;
		_bitmapName = bitmapName;
		this.setMargin(_bitmapHeight/2, _bitmapHeight/3, _bitmapHeight/2, _bitmapHeight/3);
		adaptFocusImage(true);
	}
	
	/**
	 * Changes the base name of the bitmap.
	 * @param bitmapName The base name of the bitmap without the .png suffix.
	 */
	public void setBitmapName(String bitmapName)
	{
		_bitmapName = bitmapName;
		Bitmap b = getBitmap(bitmapName + (isEnabled() ? "" : "_t50" ) + ".png", _bitmapHeight);
		setImage(b);
		setFocusImage(b);
		adaptFocusImage(isEnabled());
	}
	
	/**
	 * Returns the bitmap
	 * @param bitmapName The resource name of the bitmap.
	 * @param height The desired height of the bitmap.
	 */
	private static Bitmap getBitmap(String bitmapName, int height)
	{
		return UIUtil.getScaledBitmap(bitmapName, height);
	}
	
	/**
	 * Adapts the bitmap for the focus.
	 */
	private void adaptFocusImage(boolean isEnabled)
	{
		if (_bitmapName.endsWith("dark")) {
			String name = _bitmapName.substring(0, _bitmapName.length() - 4) +
					"light" + (isEnabled ? "" : "_t50") + ".png";
			setFocusImage(UIUtil.getScaledBitmap(name, _bitmapHeight));
		}
	}
	
	/**
	 * Enabled or disabled the button.
	 */
	public void setEnabled(boolean enable)
	{
		// change between semi-transparent and solid bitmap
		if (enable != isEnabled()) {
			if (enable) {
				Bitmap b = getBitmap(_bitmapName + ".png", _bitmapHeight);
				setImage(b);
				setFocusImage(b);
				adaptFocusImage(true);
			} else {
				Bitmap b = getBitmap(_bitmapName + "_t50.png", _bitmapHeight);
				setImage(b);
				setFocusImage(b);
				adaptFocusImage(false);
			}
		}
		super.setEnabled(enable);
	}
	
	/**
	 * Returns the preferred width.
	 */
    public int getPreferredWidth() {
    	int w = super.getPreferredWidth();
    	int h = super.getPreferredHeight();
    	return Math.max(w,  h);
    }
    
    /**
     * Returns the preferred height.
     */
    public int getPreferredHeight() {
    	int w = super.getPreferredWidth();
    	int h = super.getPreferredHeight();
    	return Math.max(w,  h);
    }
    
    /**
     * Calculates the layout. Header buttons are always square.
     */
    protected void layout( int width, int height ) {
    	int w = super.getPreferredWidth();
    	int h = super.getPreferredHeight();
    	int m = Math.max(w,  h);
        setExtent(m, m);
    }
    
	/**
	 * Paint this button.
	 */
    protected void paint( Graphics g ) {
    	if (isFocus()) {
    		XYRect rect = new XYRect(0, 0, _bitmapHeight, _bitmapHeight);
    		UIUtil.getFocusBackground().draw(g, rect);
    	}
        super.paint(g);
    }
    
    /**
     * Reacts on touch event
     */
    protected boolean touchEvent( TouchEvent message )
    {
        switch( message.getEvent() ) {
        	case TouchEvent.UNCLICK:
        		break;
            case TouchEvent.CLICK:
                clickButton();
                return true;
        }
        return super.touchEvent( message );
    }
}