/*
 *  Licensed Materials - Property of IBM
 *  5725-G92 (C) Copyright IBM Corp. 2011, 2013. All Rights Reserved.
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

package com.ibm.appcenter.ui.widget;

import net.rim.device.api.system.Bitmap;

import com.ibm.appcenter.ui.UIUtil;
import com.ibm.appcenter.ui.widget.advancedui.ListStyleButtonField;

public class EnhancedListStyleButtonField extends ListStyleButtonField
{
	private Bitmap _normalIcon;
	private Bitmap _disabledIcon;
	private Bitmap _normalActionIcon;
	private Bitmap _disabledActionIcon;
	
	/**
	 * Constructor.
	 * @param iconBaseName The base name of the icon without the .png suffix.
	 * @param iconHeight   The desired height of the icon.
	 * @param label        The label.
	 * @param actionIconBaseName The base name of the arrow icon without the .png suffix.
	 * @param actionIconHeight   The desired height of the arrow icon.
	 * @param style        The field style.
	 */
	public EnhancedListStyleButtonField(
			String iconBaseName,
			int    iconHeight,
			String label,
			String actionIconBaseName,
			int    actionIconHeight,
			long style)
	{
		this(UIUtil.getScaledBitmap(iconBaseName + ".png", iconHeight),
			 UIUtil.getScaledBitmap(iconBaseName + "_t50.png", iconHeight),
			 label,
			 UIUtil.getScaledBitmap(actionIconBaseName + ".png", actionIconHeight),
			 UIUtil.getScaledBitmap(actionIconBaseName + "_t50.png", actionIconHeight),
			 style);
	}
			
	/**
	 * Constructor.
	 * @param icon               The icon before the text.
	 * @param disabledIcon       The icon before the text, when disabled.
	 * @param label              The label.
	 * @param actionIcon         The arrow icon at the right.
	 * @param disabledActionIcon The arrow icon when disabled.
	 * @param style              The field style.
	 */
	public EnhancedListStyleButtonField(
			Bitmap icon, 
			Bitmap disabledIcon,
			String label, 
			Bitmap actionIcon,
			Bitmap disabledActionIcon,
			long style )
	{
		super(icon, label, actionIcon, style);
		_normalIcon = icon;
		_disabledIcon = disabledIcon;
		_normalActionIcon = actionIcon;
		_disabledActionIcon = disabledActionIcon;
	}
	
	public void setEnabled(boolean enable)
	{
		if (enable != isEnabled()) {
			if (enable) {
				_leftIcon = _normalIcon;
				_actionIcon = _normalActionIcon;
			} else {
				_leftIcon = _disabledIcon;
				_actionIcon = _disabledActionIcon;
			}
		}
		super.setEnabled(enable);
	}
	
	/**
	 * Change the colors of all list style buttons.
	 * @param backgroundColor
	 * @param borderColor
	 * @param backgroundFocusColor
	 * @param textNormalColor
	 * @param textDisabledColor
	 */
    public static void setColors(
    		int backgroundColor,
    		int borderColor,
    		int backgroundFocusColor,
    		int textNormalColor,
    		int textDisabledColor)
    {
    	COLOR_BACKGROUND = backgroundColor;
    	COLOR_BORDER = borderColor;
    	COLOR_BACKGROUND_FOCUS = backgroundFocusColor;
    	COLOR_TEXT_NORMAL = textNormalColor;
    	COLOR_TEXT_DISABLED = textDisabledColor;
    }
}
