/*
 *  Licensed Materials - Property of IBM
 *  5725-G92 (C) Copyright IBM Corp. 2011, 2013. All Rights Reserved.
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

package com.ibm.appcenter.ui.widget;

import com.ibm.appcenter.ui.widget.advancedui.TwoColumnField;

import net.rim.device.api.ui.Field;

/**
 * An enhanced version of the TwoColumnField that allows to
 * ensure that all right fields have the same width.
 */
public class EnhancedTwoColumnField extends TwoColumnField
{
	private int _commonRightWidth;
	
	/**
	 * Constructor.
	 * @param leftField The left field.
	 * @param rightField The right field.
	 * @param style The style.
	 */
    public EnhancedTwoColumnField(Field leftField, Field rightField, long style ) 
    {
    	super(leftField, rightField, style);
    }
    
    /**
     * Lays out the the right field and then return the width that it would like to have.
     */
    public int layoutRight( int width, int height )
    {
        layoutChild(getRightField(), width, height );
        return getRightField().getWidth();
    }
    
    /**
     * Sets the width that should be used for all right fields.
     * @param width The width.
     */
    void setCommonRightWidth(int width)
    {
    	_commonRightWidth = width;
    }
    
    /**
     * Adapts the field after layout.
     */
    protected void adaptChild(Field c, boolean isLeft)
    {
    	if (!isLeft) {
    		c.setMinimalWidth(_commonRightWidth);
    	}
    }
}
