/*
 *  Licensed Materials - Property of IBM
 *  5725-G92 (C) Copyright IBM Corp. 2011, 2013. All Rights Reserved.
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

package com.ibm.appcenter.ui.widget;

import net.rim.device.api.ui.Field;

import com.ibm.appcenter.ui.widget.advancedui.TwoColumnFieldManager;


/**
 * An enhanced version of the TwoColumnFieldManager that allows to
 * ensure that all right fields have the same width.
 */
public class EnhancedTwoColumnFieldManager
extends TwoColumnFieldManager
{
	/**
	 * Constructor.
	 * @param maxLeftColumnWidth Maximal width of the left column.
	 * @param style The style.
	 */
    public EnhancedTwoColumnFieldManager(int maxLeftColumnWidth, long style)
    {
    	super(maxLeftColumnWidth, style);
    }
    
    /**
     * Allows to adapt the right column.
     * @param width The expected width of the right column.
     * @param height The expected height of the right column.
     */
    protected void adaptRight(int width, int height)
    {
        int numFields = getFieldCount();
        int maxRightFieldWidth = 0;
        for( int i = 0; i < numFields; i++ ) {
            Field currentField = getField( i );
            if( currentField instanceof EnhancedTwoColumnField ) {
               int currentRightFieldWidth = ((EnhancedTwoColumnField) currentField).layoutRight(width, height);
               maxRightFieldWidth = Math.max(currentRightFieldWidth, maxRightFieldWidth);
           }
        }
        
        for( int i = 0; i < numFields; i++ ) {
            Field currentField = getField( i );
            if( currentField instanceof EnhancedTwoColumnField ) {
               ((EnhancedTwoColumnField) currentField).setCommonRightWidth(maxRightFieldWidth);
           }
        }
    }

}
