/*
 *  Licensed Materials - Property of IBM
 *  5725-G92 (C) Copyright IBM Corp. 2011, 2013. All Rights Reserved.
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

package com.ibm.appcenter.ui.widget;

import com.ibm.appcenter.i18n.I18N;
import com.ibm.appcenter.i18n.I18NResource;
import com.ibm.appcenter.ui.UIUtil;

import net.rim.device.api.ui.Field;
import net.rim.device.api.ui.FieldChangeListener;
import net.rim.device.api.ui.Manager;
import net.rim.device.api.ui.UiApplication;
import net.rim.device.api.ui.component.BasicEditField;
import net.rim.device.api.ui.component.EditField;
import net.rim.device.api.ui.container.HorizontalFieldManager;
import net.rim.device.api.ui.decor.Background;

public abstract class HeaderLine
extends HorizontalFieldManager
implements I18NResource
{
	private static final int SYSTEM_STYLE_SHIFT = 32;
	
	private Background _background = UIUtil.getTopHeaderBackground();
	private HeaderButtonField _backButton;
	private HeaderButtonField _refreshButton;
	private HeaderButtonField _toolsButton;
	private HeaderButtonField _searchIcon;
	private HeaderButtonField _sortIcon;
	private HeaderButtonField _updownButton;
	private EditField _searchField;
	private String _oldSearchText = "";
	private int _searchScope = 0;
	private int _sortScope = 0;
	private boolean _blackOnWhite = !UIUtil.isTopHeaderNeedsLightIcons();
		
	public HeaderLine(boolean showBackButton,
					  boolean showSearch,
					  boolean showSearchScope,
					  boolean showUpDown,
					  boolean showRefreshButton)
	{
		super(Manager.USE_ALL_WIDTH);
		setBackground(_background);
		int d = 0;
		String key = (isBlackOnWhite() ? "dark" : "light");
		if (showBackButton) {
			_backButton = new HeaderButtonField("back" + key, Field.FIELD_VCENTER);
			d = _backButton.getMarginTop();
			_backButton.setChangeListener(new FieldChangeListener() {
				public void fieldChanged(Field field, int context) {
					onBack();				
				}});
			add(_backButton);
		}
		
		if (showUpDown) {
			_updownButton = new HeaderButtonField("arrowup" + key, Field.FIELD_VCENTER);
			d = _updownButton.getMarginTop();
			_updownButton.setChangeListener(new FieldChangeListener() {
				public void fieldChanged(Field field, int context) {
					onUpDown();				
				}});
			add(_updownButton);
		}
		if (showSearch) {
			if (showSearchScope) {
				_searchIcon = new HeaderButtonField("magnification" + key, Field.FIELD_VCENTER);
				d = _searchIcon.getMarginTop();
				_searchIcon.setMargin(d,
									  (isLeftToRightOwnVersion() ? 0 : 3*d),  // = right margin
									  d, 
									  (isLeftToRightOwnVersion() ? 3*d : 0)); // = left margin
				_searchIcon.setChangeListener(new FieldChangeListener() {
					public void fieldChanged(Field field, int context) {
						onSearchScope();				
					}});
				add(_searchIcon);
			}
			_searchField = new EnhancedEditField(
        		"",
        		"",
        		I18N.getString(I18NResource.HeaderLineSearchHint),
        		BasicEditField.DEFAULT_MAXCHARS,
        		EditField.USE_ALL_WIDTH | EditField.EDITABLE | EditField.NO_NEWLINE | 
        			EditField.FIELD_VCENTER | EditField.NON_SPELLCHECKABLE | EditField.NO_LEARNING |
        			EditField.NO_COMPLEX_INPUT);
			if (showSearchScope) {
				_searchField.setMargin(_searchField.getMarginTop(),
				  			   	   	   (isLeftToRightOwnVersion() ? 3*d : 0), // right margin 
				  			   	   	   _searchField.getMarginBottom(), 
				  			   	   	   (isLeftToRightOwnVersion() ? 0 : 3*d)); // left margin
			} else {
				_searchField.setMargin(_searchField.getMarginTop(),
									   3*d, 
									   _searchField.getMarginBottom(), 
									   3*d);
			}
			_searchField.setChangeListener(new FieldChangeListener() {
				public void fieldChanged(Field field, int context) {
					if (_oldSearchText == null || !_oldSearchText.equals(_searchField.getText())) {
						_oldSearchText = _searchField.getText();
						onSearch(_searchField.getText(), getSearchScope(), getSortScope());	
					}
				}});
			add(_searchField);
			_sortIcon = new HeaderButtonField("sort" + key, Field.FIELD_VCENTER);
			d = _sortIcon.getMarginTop();
			_sortIcon.setMargin(d,
								(isLeftToRightOwnVersion() ? 0 : 0),  // = right margin
								d, 
								(isLeftToRightOwnVersion() ? 0 : 0)); // = left margin
			_sortIcon.setChangeListener(new FieldChangeListener() {
				public void fieldChanged(Field field, int context) {
					onSortScope();				
				}});
			add(_sortIcon);
		}
		
		if (showRefreshButton) {
			_refreshButton = new HeaderButtonField("refresh" + key, Field.FIELD_VCENTER);
			_refreshButton.setChangeListener(new FieldChangeListener() {
				public void fieldChanged(Field field, int context) {
					onRefresh();
				}});
			add(_refreshButton);
		}
		// not yet used
		_toolsButton = null;
	}
	
	/**
	 * Layout the container.
	 * @param maxWidth The maximal width available for the header line.
	 * @param maxHeight The maximal height available for the header line.
	 */
	protected void sublayout(int maxWidth, int maxHeight)
	{
        int maxChildHeight = 0;

        Field[] leftFields = null;
        Field[] rightFields = null;
        Field[] centerFields = null;
        if (isLeftToRightOwnVersion()) {
        	leftFields = new Field[1];
        	rightFields = new Field[3];
        	centerFields = new Field[3];
        	leftFields[0] = _backButton;
        	rightFields[0] = _sortIcon;
        	rightFields[1] = _toolsButton;
        	rightFields[2] = _refreshButton;
        	centerFields[0] = _updownButton;
        	centerFields[1] = _searchIcon;
        	centerFields[2] = _searchField;
        } else {
        	leftFields = new Field[3];
        	rightFields = new Field[1];
        	centerFields = new Field[3];
        	leftFields[0] = _sortIcon;
        	leftFields[1] = _toolsButton;
        	leftFields[2] = _refreshButton;
        	rightFields[0] = _backButton;
        	centerFields[0] = _searchField;
        	centerFields[1] = _searchIcon;
        	centerFields[2] = _updownButton;
        }
        
        // calculate availableWidth without all margins
        int availableWidth = maxWidth;
        int m = 0;
        for (int i = 0; i < leftFields.length; i++) {
        	Field f = leftFields[i];
        	if (f == null) continue;
        	availableWidth -= Math.max(m, f.getMarginLeft());
        	m = f.getMarginRight();
        }
        availableWidth -= m;
        
        m = 0;
        for (int i = 0; i < rightFields.length; i++) {
        	Field f = rightFields[i];
        	if (f == null) continue;
        	availableWidth -= Math.max(m, f.getMarginRight());
        	m = f.getMarginLeft();
        }
        availableWidth -= m;

        m = 0;
        for (int i = 0; i < centerFields.length; i++) {
        	Field f = centerFields[i];
        	if (f == null) continue;
        	availableWidth -= Math.max(m, f.getMarginLeft());
        	m = f.getMarginRight();
        }
        availableWidth -= m;
        
        // layout the children at the left side
        for (int i = 0; i < leftFields.length; i++) {
        	Field f = leftFields[i];
        	if (f == null) continue;
        	layoutChild(f, availableWidth, maxHeight - f.getMarginTop() - f.getMarginBottom());
        	maxChildHeight = Math.max( maxChildHeight, f.getMarginTop() + f.getHeight() + f.getMarginBottom() );
        	availableWidth -= f.getWidth();
        }

        // layout the children at the right side
        for (int i = 0; i < rightFields.length; i++) {
        	Field f = rightFields[i];
        	if (f == null) continue;
        	layoutChild(f, availableWidth, maxHeight - f.getMarginTop() - f.getMarginBottom());
        	maxChildHeight = Math.max( maxChildHeight, f.getMarginTop() + f.getHeight() + f.getMarginBottom() );
        	availableWidth -= f.getWidth();
        }

        // layout the children at the center side.
        // Need to ensure that the _searchField is handled last, since it is the only stretchable field.
        if (isLeftToRightOwnVersion()) {
        	for (int i = 0; i < centerFields.length; i++) {
            	Field f = centerFields[i];
            	if (f == null) continue;
            	layoutChild(f, availableWidth, maxHeight - f.getMarginTop() - f.getMarginBottom());
            	maxChildHeight = Math.max( maxChildHeight, f.getMarginTop() + f.getHeight() + f.getMarginBottom() );
            	availableWidth -= f.getWidth();
            }
        } else {
        	for (int i = centerFields.length - 1; i >= 0; i--) {
            	Field f = centerFields[i];
            	if (f == null) continue;
            	layoutChild(f, availableWidth, maxHeight - f.getMarginTop() - f.getMarginBottom());
            	maxChildHeight = Math.max( maxChildHeight, f.getMarginTop() + f.getHeight() + f.getMarginBottom() );
            	availableWidth -= f.getWidth();
            }
        }
        
        maxHeight = maxChildHeight;
        
        // position fields at the left side
        int x1 = 0;
        m = 0;
        for (int i = 0; i < leftFields.length; i++) {
        	Field f = leftFields[i];
        	if (f == null) continue;
        	x1 = x1 + Math.max(m, f.getMarginLeft());
    		setPositionChild(f, x1, getFieldY(f, maxHeight));
    		x1 += f.getWidth();
    		m = f.getMarginRight();
        }
        
        // position fields at the right side
        int x2 = maxWidth;
        m = 0;
        for (int i = rightFields.length -1; i >= 0; i--) {
        	Field f = rightFields[i];
        	if (f == null) continue;
        	x2 = x2 - f.getWidth() - Math.max(m, f.getMarginRight());
    		setPositionChild(f, x2, getFieldY(f, maxHeight));
    		m = f.getMarginLeft();
        }
        
        // position fields in the center
        int centerwidth = 0;
        m = 0;
        for (int i = 0; i < centerFields.length; i++) {
        	Field f = centerFields[i];
        	if (f == null) continue;
        	centerwidth += Math.max(m, f.getMarginLeft());
    		centerwidth += f.getWidth();
    		m = f.getMarginRight();
        }
        centerwidth += m;
        int x3 = (x1 + x2 - centerwidth)/2;
        for (int i = 0; i < centerFields.length; i++) {
        	Field f = centerFields[i];
        	if (f == null) continue;
        	x3 = x3 + Math.max(m, f.getMarginLeft());
    		setPositionChild(f, x3, getFieldY(f, maxHeight));
    		x3 += f.getWidth();
    		m = f.getMarginRight();
        }

        setExtent(maxWidth, maxHeight);
	}
	
	/**
	 * Calculates the y position of a field
	 * considering its vertical alignment.
	 * @param field The field.
	 * @param height The available height.
	 * @return The y position relative to the origin,
	 *   i.e. returning 0 would be top of the area.
	 */
    private int getFieldY( Field field, int height )
    {
        switch( (int)( ( field.getStyle() & FIELD_VALIGN_MASK ) >> SYSTEM_STYLE_SHIFT ) ) {
            case (int)( FIELD_BOTTOM >> SYSTEM_STYLE_SHIFT ):
                return height - field.getHeight() - field.getMarginBottom();
            case (int)( FIELD_VCENTER >> SYSTEM_STYLE_SHIFT ):
                return field.getMarginTop() + ( height - field.getMarginTop() - field.getHeight() - field.getMarginBottom() ) / 2;
            default:
                return field.getMarginTop();
        }
    }
    
    /**
     * Called when the back button was pressed.
     */
	protected abstract void onBack();

	/**
	 * Called when the refresh button was pressed.
	 */
	protected abstract void onRefresh();
	
	/**
	 * Called when the up down button was pressed.
	 */
	protected void onUpDown()
	{
	}
	
	/**
	 * Called when the search scope button was pressed.
	 */
	protected void onSearchScope()
	{
		String[] choices = getSearchScopeDisplayNames();
		int[] values = getSearchScopes();
        
        SpinnerPopup popup = new SpinnerPopup(I18N.getString(HeaderLineSelectSearchScope), choices, values) {
            protected void onSelect(int choice)
            {
            	if (choice != _searchScope) {
            		_searchScope = choice;
            		onSearch(_searchField.getText(), getSearchScope(), getSortScope());
            	}
            }
        };
        popup.setSelectedChoice(getSearchScope());
        UiApplication app = UiApplication.getUiApplication();
        app.pushScreen(popup);
	}
	
	/**
	 * Called when the sort scope button was pressed.
	 */
	protected void onSortScope()
	{
		if (UIUtil.isAscendingDescendingSortAllowed())
			onSortScopeAscendingDescending();
		else
			onSortScopeUnidirectional();
	}
	
	/**
	 * Called when the sort scope button was pressed.
	 * This variant allows ascending and descending sort.
	 */
	private void onSortScopeAscendingDescending()
	{
		String[] choices = getSortScopeDisplayNames(true);
		int[] values = getSortScopes(true);
		
		String[] choices2 = new String[] { 
				I18N.getString(HeaderLineAscendingOrdering), 
				I18N.getString(HeaderLineDescendingOrdering) };
		int[][] valueMap = new int[choices.length][2];
		
		for (int i = 0; i < values.length; i++) {
			valueMap[i][0] = values[i];
			valueMap[i][1] = values[i] + 1;
		}
		
        DoubleSpinnerPopup popup = new DoubleSpinnerPopup(I18N.getString(HeaderLineSelectSortScope), choices, choices2, valueMap) {
            protected void onSelect(int choice)
            {
            	if (choice != _sortScope) {
            		_sortScope = choice;
            		onSearch(_searchField.getText(), getSearchScope(), getSortScope());
            	}
            }
        };
        popup.setSelectedChoice(getSortScope());
        UiApplication app = UiApplication.getUiApplication();
        app.pushScreen(popup);
	}
	
	/**
	 * Called when the sort scope button was pressed.
	 * This variant allows only unidirectional sort.
	 */
	private void onSortScopeUnidirectional()
	{
		String[] choices = getSortScopeDisplayNames(false);
		int[] values = getSortScopes(false);
				
        SpinnerPopup popup = new SpinnerPopup(I18N.getString(HeaderLineSelectSortScope), choices, values) {
            protected void onSelect(int choice)
            {
            	if (choice != _sortScope) {
            		_sortScope = choice;
            		onSearch(_searchField.getText(), getSearchScope(), getSortScope());
            	}
            }
        };
        popup.setSelectedChoice(getSortScope());
        UiApplication app = UiApplication.getUiApplication();
        app.pushScreen(popup);
	}
	
	/**
	 * Called when text was entered in the search.
	 * This is called for every character that was
	 * entered.
	 * @param text The entire text that was entered.
	 * @param searchScope The search scope.
	 * @param sortMode    The ordering mode.
	 */
	protected abstract void onSearch(String text, int searchScope, int sortMode);

	
	/**
	 * Returns the display names of the search scopes.
	 * The returned array must have the same size
	 * as the returned array of {@link #getSearchScopes()}. 
	 */
	protected String[] getSearchScopeDisplayNames()
	{
		return new String[] { "dummy" };
	}
	
	/**
	 * Returns the search scopes.
	 * The returned array must have the same size
	 * as the returned array of {@link #getSearchScopeDisplayNames()}. 
	 */
	protected int[] getSearchScopes()
	{
		return new int[] { 0 };
	}
	
	/**
	 * Returns the current search scope.
	 */
	public int getSearchScope()
	{
		return _searchScope;
	}
	
	/**
	 * Sets the current search scope.
	 */
	public void setSearchScope(int scope)
	{
		_searchScope = scope;
	}
	
	/**
	 * Returns the display names of the sort scopes.
	 * The returned array must have the same size
	 * as the returned array of {@link #getSortScopes()}. 
	 * @param allowAscDesc Whether ascending and descending sort is allowed.
	 */
	protected String[] getSortScopeDisplayNames(boolean allowAscDesc)
	{
		return new String[] { "dummy" };
	}
	
	/**
	 * Returns the sort scopes.
	 * The returned array must have the same size
	 * as the returned array of {@link #getSortScopeDisplayNames()}.
	 * All returned values must be even (odd numbers are reserved
	 * for the corresponding reverse order). 
	 * @param allowAscDesc Whether ascending and descending sort is allowed.
	 */
	protected int[] getSortScopes(boolean allowAscDesc)
	{
		return new int[] { 0 };
	}
	
	/**
	 * Returns the current sort scope.
	 */
	public int getSortScope()
	{
		return _sortScope;
	}
	
	/**
	 * Sets the current sort scope.
	 */
	public void setSortScope(int scope)
	{
		_sortScope = scope;
	}
	
	/**
	 * Sets the black on white mode. Icons are dark grey and the
	 * background is supposed to be light. Otherwise icons are
	 * near white and the background is supposed to be dark.
	 * @param enable Whether to enable the back on white mode.
	 *
	public void setBlackOnWhite(boolean enable)
	{
		if (enable == _blackOnWhite)
			return;
		_blackOnWhite = enable;
		
		if (_backButton != null) {
			_backButton.setBitmapName("arrow" + (isLeftToRightOwnVersion() ? "left" : "right") + (enable ? "dark" : "light"));
		}
		if (_searchIcon != null) {
			_searchIcon.setBitmapName("magnification" + (enable ? "dark" : "light"));
		}
		if (_sortIcon != null) {
			_sortIcon.setBitmapName("sort" + (enable ? "dark" : "light"));
		}
		if (_refreshButton != null) {
			_refreshButton.setBitmapName("refresh" + (enable ? "dark" : "light"));
		}
		if (_toolsButton != null) {
			_toolsButton.setBitmapName("tools" + (enable ? "dark" : "light"));
		}
	}
	 */
	
	/**
	 * Returns whether the black on white mode is enabled.
	 */
	public boolean isBlackOnWhite()
	{
		return _blackOnWhite;
	}
	
	/**
	 * Enable or disable the header line.
	 */
	public void setEnabled(boolean enable)
	{
		if (_backButton != null) {
			_backButton.setEnabled(true);
		}
		if (_searchIcon != null) {
			_searchIcon.setEnabled(enable);
		}
		if (_sortIcon != null) {
			_sortIcon.setEnabled(enable);
		}
		if (_searchField != null) {
			_searchField.setEnabled(enable);
		}
		if (_updownButton != null) {
			_updownButton.setEnabled(enable);
		}
		if (_refreshButton != null) {
			boolean old = _refreshButton.isEnabled();
			_refreshButton.setEnabled(enable);
			// ensure that we don't lose the focus
			if (old == false && enable == true) {
				_refreshButton.setFocus();
				
			}
		}
		if (_toolsButton != null) {
			_toolsButton.setEnabled(enable);
		}
		// super.setEnabled(enable);
	}
	
	/**
	 * Set the initial focus.
	 */
	public void setInitialFocus()
	{
		if (_refreshButton != null) {
			_refreshButton.setEnabled(true);
			_refreshButton.setFocus();
		}
	}
	
	/**
	 * Set the safe focus.
	 */
	public void setSafeFocus()
	{
		if (_backButton != null) {
			_backButton.setEnabled(true);
			_backButton.setFocus();
		}
	}
	
	/**
	 * Switches the visual state of the up down button.
	 */
	public void setUpDownButtonMode(boolean up)
	{
		if (_updownButton == null) return;
		String bitmapName = (up ? "arrowup" : "arrowdown") + (isBlackOnWhite() ? "dark" : "light");
		_updownButton.setBitmapName(bitmapName);
	}
	
	/**
	 * Returns whether we are in LTR or RTL mode.
	 */
	public boolean isLeftToRightOwnVersion()
	{
		return true;
	}
}
