/*
 *  Licensed Materials - Property of IBM
 *  5725-G92 (C) Copyright IBM Corp. 2011, 2013. All Rights Reserved.
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

package com.ibm.appcenter.ui.widget;

import com.ibm.appcenter.ui.UIUtil;

import net.rim.device.api.ui.Field;
import net.rim.device.api.ui.Manager;

public class LeadingTrailingManager extends Manager
{
	Field _left = null;
	Field _right = null;
	
	/**
	 * Constructor.
	 */
	public LeadingTrailingManager()
	{
		this(0);
	}
	
	/**
	 * Constructor.
	 * @param style The style.
	 */
	public LeadingTrailingManager(long style)
	{
		super(style);
	}

	/**
	 * Calculates the layout.
	 */
	protected void sublayout(int width, int height) {
		
		int availableWidth = width;
		int maxHeight = 0;
		int numFields = getFieldCount();
		
		// substract the margins from the available width
		for( int i = 0; i < numFields; i++ ) {
            Field f = getField(i);
            availableWidth -= f.getMarginLeft();
            availableWidth -= f.getMarginRight();
		}
		
		// layout all trailing children
		int w = availableWidth / 2;
		for( int i = 0; i < numFields; i++ ) {
            Field f = getField(i);
            if ((f.getStyle() & Field.FIELD_TRAILING) == Field.FIELD_TRAILING) {
            	layoutChild(f, w, height);
            	availableWidth -= f.getWidth();
            	maxHeight = Math.max(maxHeight, f.getHeight());
            }
		}
		
		// layout all leading and center children
		w = availableWidth;
		for( int i = 0; i < numFields; i++ ) {
            Field f = getField(i);
            if ((f.getStyle() & Field.FIELD_TRAILING) != Field.FIELD_TRAILING) {
            	layoutChild(f, w, height);
            	maxHeight = Math.max(maxHeight, f.getHeight());
            }
		}
		
		// Sets the size of this
		setExtent(width, maxHeight);
		
		// place all children
		for( int i = 0; i < numFields; i++ ) {
            Field f = getField(i);
            if (UIUtil.isLeftToRight(this)) {
            	if ((f.getStyle() & Field.FIELD_TRAILING) == Field.FIELD_TRAILING) {
            		// trailing
            		int x = width - f.getMarginRight() - f.getWidth();
            		setPositionChild(f, x, 0);
            	} else if ((f.getStyle() & Field.FIELD_HCENTER) == Field.FIELD_HCENTER) {
            		// center
            		int fw = f.getWidth() + f.getMarginLeft() + f.getMarginRight();
            		int x = (width - fw) / 2;
            		setPositionChild(f, x, 0);
            	} else {
            		// leading
            		int x = f.getMarginLeft();
            		setPositionChild(f, x, 0);
            	}
            } else {
            	if ((f.getStyle() & Field.FIELD_TRAILING) == Field.FIELD_TRAILING) {
            		// trailing
            		int x = f.getMarginLeft();
            		setPositionChild(f, x, 0);
            	} else if ((f.getStyle() & Field.FIELD_HCENTER) == Field.FIELD_HCENTER) {
            		// center
            		int fw = f.getWidth() + f.getMarginLeft() + f.getMarginRight();
            		int x = (width - fw) / 2;
            		setPositionChild(f, x, 0);
            	} else {
            		// leading
            		int x = width - f.getMarginRight() - f.getWidth();
            		setPositionChild(f, x, 0);
            	}
            }
		}
	}
}
