/*
 *  Licensed Materials - Property of IBM
 *  5725-G92 (C) Copyright IBM Corp. 2011, 2013. All Rights Reserved.
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

package com.ibm.appcenter.ui.widget;

import net.rim.device.api.system.Bitmap;
import net.rim.device.api.ui.container.HorizontalFieldManager;

import com.ibm.appcenter.ui.UIUtil;
import com.ibm.appcenter.ui.widget.advancedui.BitmapButtonField;

/**
 * A non-interactive field that displays 5 stars.
 */
public class RatingDisplayField extends HorizontalFieldManager
{
	private double _rating;
	private BitmapButtonField[] _fields;
	
	private static Bitmap _emptyStar = null;
	private static Bitmap _fullStar = null;
	private static Bitmap _halfStar = null;
	   
	/**
	 * Constructor.
	 * Notice that we assume that we use all rating fields with the
	 * same star size and arrow color.
	 */
	public RatingDisplayField(int starSize, long style)
	{
		super(style);
		
		if (_emptyStar == null) {
			_emptyStar = UIUtil.getScaledBitmap("emptystar.png", starSize);
		}
		if (_fullStar == null) {
			_fullStar = UIUtil.getScaledBitmap("fullstar.png", starSize);
		}
		if (_halfStar == null) {
			_halfStar = UIUtil.getScaledBitmap("halfstar.png", starSize);
		}
		_fields = new BitmapButtonField[5];
		for (int i = 0; i < _fields.length; i++) {
			_fields[i] = new BitmapButtonField(_emptyStar, _emptyStar);
			_fields[i].setEditable(false);
			add(_fields[i]);
		}
	}
	
	/**
	 * Changes the rating of this field.
	 * @param rating The rating.
	 */
	public void setRating(double rating)
	{
		if (rating != _rating) {
			_rating = rating;
			
			Bitmap bm;
	    	for (int i = 0; i < _fields.length; i++) {
	    		if (rating < 1.0/3) {
	    			bm = _emptyStar;
	    		} else if (rating < 2.0/3) {
	    			bm = _halfStar;
	    		} else {
	    			bm = _fullStar;
	    		}
	    		_fields[i].setImage(bm);
	    		_fields[i].setFocusImage(bm);
	    		rating = rating - 1;
	    	}
			invalidate();
		}
	}
}