/*
 *  Licensed Materials - Property of IBM
 *  5725-G92 (C) Copyright IBM Corp. 2011, 2013. All Rights Reserved.
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

package com.ibm.appcenter.ui.widget;

import java.util.Timer;
import java.util.TimerTask;

import net.rim.device.api.ui.component.Status;

import com.ibm.appcenter.i18n.I18N;
import com.ibm.appcenter.i18n.I18NResource;
import com.ibm.appcenter.ui.AppCenterScreen;
import com.ibm.appcenter.ui.UIUtil;
import com.ibm.appcenter.util.BusyIndicator;
import com.ibm.appcenter.util.Log;

/**
 * Manager that handles a popup that indicates
 * to wait because the application is busy.
 * The popup does not appear immediately but
 * after a delay, so that very fast operations
 * never start the popup.
 */
public class WaitPopupManager 
implements BusyIndicator 
{
	private int _delay;
	private boolean _busy;
	private AppCenterScreen _screen;
	private Timer _timer;
	
	/**
	 * Constructor.
	 * @param screen The associated screen.
	 * @param delay The delay until the popup is shown.
	 */
	public WaitPopupManager(AppCenterScreen screen, int delay)
	{
		_screen = screen;
		_delay = delay;
	}
	
	/**
	 * Sets whether the indicator displays
	 * the busy or idle state.
	 * @param busy If true, then busy state.
	 */
	public synchronized void setBusy(boolean busy) {
		if (isBusy() != busy) {
			Log.log("WaitPopupManager setBusy " + busy);
			if (_screen.isDisplayed()) {
				if (busy) {
					_timer = new Timer();
					_timer.schedule(new TimerTask() {
						public void run() {
							if (_busy) {
								UIUtil.invokeInEventThread(new Runnable() {
									public void run() {
										if (_busy) {
											Log.log("WaitPopupManager " + hashCode() + " Show please wait");
											Status.show(I18N.getString(I18NResource.WaitPopupManagerPleaseWaitMsg));
										}
									}});
							}
						}}, _delay);
				} else {
					Log.log("WaitPopupManager " + hashCode() + " canceled showing please wait");
					if (_timer != null) {
						_timer.cancel();
						_timer = null;
					}
				}
			}
		}
		_screen.setBusy(busy);
		_busy = busy;
	}

	/**
	 * Returns whether the indicator displays
	 * the busy or idle state.
	 */
	public boolean isBusy() {
		return _busy;
	}

	/**
	 * Shows the busy indicator immediately without waiting.
	 * Also sets the state to busy.
	 */
	public void setBusyImmediate()
	{
		Log.log("WaitPopupManager setBusyImmediate");
		_screen.setBusy(true);
		_busy = true;
		UIUtil.invokeInEventThread(new Runnable() {
			public void run() {
				Log.log("WaitPopupManager " + hashCode() + " Show please wait");
				Status.show(I18N.getString(I18NResource.WaitPopupManagerPleaseWaitMsg));
			}});
	}
}
