@echo off
@REM WebSphere Application Server liberty launch script
@REM
@REM Copyright IBM Corp. 2011
@REM The source code for this program is not published or other-
@REM wise divested of its trade secrets, irrespective of what has
@REM been deposited with the U.S. Copyright Office.
@REM 
@REM ----------------------------------------------------------------------------
@REM
@REM To customize the use of this script (for example with /etc/init.d system 
@REM service managers), use the following environment variables:
@REM
@REM JAVA_HOME  - The java executable is found in %JAVA_HOME%\bin
@REM
@REM JVM_ARGS   - A list of JVM command line options,
@REM              e.g. system properties or -X parameters
@REM              The value will be expanded by cmd.exe (use quotes for spaces)
@REM
@REM LOG_DIR    - The log file directory
@REM              The default value is %WLP_OUTPUT_DIR%\serverName\logs
@REM
@REM LOG_FILE   - The log file name
@REM              This log file is only used if the server is run in the
@REM              background via the start action. 
@REM              The default value is console.log
@REM
@REM WLP_USER_DIR - The user/custom configuration directory used to store
@REM              shared and server-specific configuration. 
@REM              See README.TXT for details about shared resource locations.
@REM              A server's configuration is at %WLP_USER_DIR%\servers\serverName
@REM              The default value is the usr directory in the install directory.
@REM
@REM WLP_OUTPUT_DIR - The directory containing output files for defined servers.
@REM              This directory must have both read and write permissions for
@REM              the user or users that start servers.
@REM              By default, a server's output logs and workarea are stored
@REM              in the %WLP_USER_DIR%\servers\serverName directory
@REM              (alongside configuration and applications).
@REM              If this variable is set, the output logs and workarea 
@REM              would be stored in %WLP_OUTPUT_DIR%\serverName.

setlocal enabledelayedexpansion

@REM We set enabledelayedexpansion to allow !VAR!.  Quoting rules:
@REM - Use "%VAR%" (for passing to commands) or !VAR! (for echo) to expand
@REM   variables containing unknown content.  %VAR% is substituted before
@REM   parsing, which causes problems if the value contains () or &, and it
@REM   causes the wrong value to be used if the variable is set within "if".
@REM - Use set VAR=!VAR2!, not set VAR="!VAR2!", and use "%VAR%" when testing
@REM   or passing the variable.  %0 and environment variables must be dequoted
@REM   specially to avoid issues with quotes or &.  There is no reliable way to
@REM   handle & within %*, so users should not call the script with ^&.
@REM - Use !VAR! not "%VAR%" to write to the console.  Otherwise, the quotes
@REM   will be output literally.

@REM If the user has explicitly set %errorlevel% set in their environment, then
@REM it will lose its special properties.  Reset it.
set errorlevel=

set CURRENT_DIR="%~dp0"
set CURRENT_DIR=!CURRENT_DIR:"=!
set WLP_INSTALL_DIR=!CURRENT_DIR:~0,-5!
set INVOKED="%~0"
set INVOKED=!INVOKED:"=!
set PARAMS_QUOTED=%*
call:closeDoubleQuotes PARAMS_QUOTED

@REM De-quote input environment variables.
if defined JRE_HOME set JRE_HOME=!JRE_HOME:"=!
if defined JAVA_HOME set JAVA_HOME=!JAVA_HOME:"=!
if defined LOG_DIR set LOG_DIR=!LOG_DIR:"=!
if defined LOG_FILE set LOG_FILE=!LOG_FILE:"=!
if defined WLP_USER_DIR set WLP_USER_DIR=!WLP_USER_DIR:"=!
if defined WLP_OUTPUT_DIR set WLP_OUTPUT_DIR=!WLP_OUTPUT_DIR:"=!

@REM Consume script parameters

@REM Set the action - must be present
set ACTION=%~1
if "%ACTION%" == "" set ACTION=help:usage
shift & call:shiftVar PARAMS_QUOTED

@REM Set the server name - optional
set SERVER_ARG=%1
set SERVER_NAME=%~1
if not defined SERVER_ARG (
  set SERVER_NAME=defaultServer
) else if "%SERVER_NAME%" == "" (
  set SERVER_NAME=defaultServer
  shift & call:shiftVar PARAMS_QUOTED
) else if "%SERVER_NAME:~0,1%" == "-" (
  set SERVER_NAME=defaultServer
) else (
  shift & call:shiftVar PARAMS_QUOTED
)

@REM Set JAVA_PARAMS_QUOTED.
set JAVA_AGENT=-javaagent:!WLP_INSTALL_DIR!\lib\bootstrap-agent.jar
if defined WLP_SKIP_BOOTSTRAP_AGENT set JAVA_AGENT=
set JAVA_PARAMS_QUOTED=!JVM_ARGS! "%JAVA_AGENT%" -jar "%WLP_INSTALL_DIR%\lib\ws-launch.jar"

set RC=255

@REM process the selected option...
if "help" == "%ACTION%" (
  call:help
) else if "help:usage" == "%ACTION%" (
  call:usage
) else if "version" == "%ACTION%" (
  call:version
) else if "create" == "%ACTION%" (
  call:createServer
) else if "run" == "%ACTION%" (
  call:runServer
) else if "debug" == "%ACTION%" (
  set JAVA_PARAMS_QUOTED=-Dwas.debug.mode=true -agentlib:jdwp=transport=dt_socket,server=y,suspend=y,address=7777 !JAVA_PARAMS_QUOTED!
  call:runServer
) else if "status" == "%ACTION%" (
  call:serverStatus
) else if "status:fast" == "%ACTION%" (
  call:serverStatusFast
) else if "start" == "%ACTION%" (
  call:startServer
) else if "package" == "%ACTION%" (
  call:package
) else if "stop" == "%ACTION%" (
  call:stopServer
) else if "dump" == "%ACTION%" (
  call:dump
) else if "javadump" == "%ACTION%" (
  call:javadump
) else (
  goto:actions
)

@REM
@REM THE END.
@REM -- we're really done now.
@REM EXIT /B will return to where we before calling SETLOCAL.
@REM EXIT will quit the CMD shell entirely.
@REM EXIT_ALL is required on Windows XP because it loses the errorlevel when
@REM using exit /b when this script is invoked from Java.  %COMSPEC% /c exit
@REM works but loses errorlevel when invoked from Cygwin.
@REM
if not defined EXIT_ALL set EXIT_ALL=0
if %EXIT_ALL% == 1 (
  EXIT %RC%
) else (
  EXIT /B %RC%
)
ENDLOCAL

:usage
  call:installEnv
  !JAVA_CMD_QUOTED! !JAVA_PARAMS_QUOTED! --script="%INVOKED%" --help:usage
  set RC=0
goto:eof

:actions
  call:installEnv
  !JAVA_CMD_QUOTED! !JAVA_PARAMS_QUOTED! --script="%INVOKED%" --help:actions
  set RC=0
goto:eof

:help
  call:installEnv
  !JAVA_CMD_QUOTED! !JAVA_PARAMS_QUOTED! --script="%INVOKED%" --help
  set RC=0
goto:eof

:version
  call:installEnv
  !JAVA_CMD_QUOTED! !JAVA_PARAMS_QUOTED! --version
goto:eof

:serverStatus
  call:serverEnv
  call:serverExists true
  if %RC% == 2 goto:eof

  call:clientCmdLog status

  !JAVA_CMD_QUOTED! !JAVA_PARAMS_QUOTED! "%SERVER_NAME%" --status
  set RC=%errorlevel%
goto:eof

:serverStatusFast
  call:serverEnv
  call:serverExists
  if %RC% == 2 goto:eof

  call:serverRunning
goto:eof

:createServer
  call:installEnv

  if not exist "%WLP_OUTPUT_DIR%\.logs" mkdir "%WLP_OUTPUT_DIR%\.logs"
  set CREATE_LOG=!WLP_OUTPUT_DIR!\.logs\!SERVER_NAME!-create.log

  echo !JAVA_CMD_QUOTED! !JAVA_PARAMS_QUOTED! "%SERVER_NAME%" --create !PARAMS_QUOTED! > "%CREATE_LOG%"
  !JAVA_CMD_QUOTED! !JAVA_PARAMS_QUOTED! "%SERVER_NAME%" --create !PARAMS_QUOTED! 2>> "%CREATE_LOG%"
  set RC=%errorlevel%

  if %RC% == 0 del /Q "%CREATE_LOG%" 1>nul 2>&1
goto:eof

:runServer
  call:serverEnvAndJVMOptions
  call:serverExists true
  if %RC% == 2 goto:eof

  call:serverWorkingDirectory
  !JAVA_CMD_QUOTED! !JVM_OPTIONS! !JAVA_PARAMS_QUOTED! "%SERVER_NAME%" !PARAMS_QUOTED!
  set RC=%ERRORLEVEL%
goto:eof

:startServer
  call:serverEnvAndJVMOptions
  call:serverExists true
  if %RC% == 2 goto:eof

  call:clientCmdLog start
  call:serverRunning
  if %RC% == 0 (
    !JAVA_CMD_QUOTED! !JAVA_PARAMS_QUOTED! --message:info.serverIsRunning "%SERVER_NAME%"
    set RC=1
  ) else (
    call:serverWorkingDirectory

    if not exist "%SERVER_OUTPUT_DIR%\workarea" mkdir "%SERVER_OUTPUT_DIR%\workarea"
    type nul > "%SERVER_OUTPUT_DIR%\workarea\.sLock"
    del "%SERVER_OUTPUT_DIR%\workarea\.sCommand" 2> nul

    START /B "" !JAVA_CMD_QUOTED! !JVM_OPTIONS! !JAVA_PARAMS_QUOTED! "!SERVER_NAME!" !PARAMS_QUOTED! > "%LOG_DIR%\%LOG_FILE%" 2>&1

    echo !JAVA_CMD_QUOTED! !JAVA_PARAMS_QUOTED! "!SERVER_NAME!" --status:start > "%CLIENT_CMD_LOG%"
    !JAVA_CMD_QUOTED! !JAVA_PARAMS_QUOTED! "!SERVER_NAME!" --status:start 2>> "%CLIENT_CMD_LOG%"
    set RC=!errorlevel!

    if !RC! == 0 del /Q "%CLIENT_CMD_LOG%" 1>nul 2>&1
  )
goto:eof

:stopServer
  call:serverEnv
  call:serverExists true
  if %RC% == 2 goto:eof

  call:clientCmdLog stop
  echo !JAVA_CMD_QUOTED! !JAVA_PARAMS_QUOTED! "!SERVER_NAME!" --stop !PARAMS_QUOTED! > "%CLIENT_CMD_LOG%"
  !JAVA_CMD_QUOTED! !JAVA_PARAMS_QUOTED! "!SERVER_NAME!" --stop !PARAMS_QUOTED! 2>> "%CLIENT_CMD_LOG%"
  set RC=!errorlevel!

  if !RC! == 0 del /Q "%CLIENT_CMD_LOG%" 1>nul 2>&1 
goto:eof

:package
  call:serverEnv
  call:serverExists true
  if %RC% == 2 goto:eof

  call:clientCmdLog package
  echo !JAVA_CMD_QUOTED! !JAVA_PARAMS_QUOTED! "!SERVER_NAME!" --package !PARAMS_QUOTED! > "%CLIENT_CMD_LOG%"
  !JAVA_CMD_QUOTED! !JAVA_PARAMS_QUOTED! "!SERVER_NAME!" --package !PARAMS_QUOTED! 2>> "%CLIENT_CMD_LOG%"
  set RC=%errorlevel%

  if %RC% == 0 del /Q "%CLIENT_CMD_LOG%" 1>nul 2>&1
goto:eof

:dump
  call:serverEnv
  call:serverExists true
  if %RC% == 2 goto:eof

  call:clientCmdLog dump
  echo !JAVA_CMD_QUOTED! !JAVA_PARAMS_QUOTED! "%SERVER_NAME%" --dump !PARAMS_QUOTED! > "%CLIENT_CMD_LOG%"
  !JAVA_CMD_QUOTED! !JAVA_PARAMS_QUOTED! "%SERVER_NAME%" --dump !PARAMS_QUOTED! 2>> "%CLIENT_CMD_LOG%" 
  set RC=%errorlevel%

  if %RC% == 0 del /Q "%CLIENT_CMD_LOG%" 1>nul 2>&1
goto:eof

:javadump
  call:serverEnv
  call:serverExists true
  if %RC% == 2 goto:eof

  call:clientCmdLog javadump
  echo !JAVA_CMD_QUOTED! !JAVA_PARAMS_QUOTED! "%SERVER_NAME%" --javadump !PARAMS_QUOTED! > "%CLIENT_CMD_LOG%"
  !JAVA_CMD_QUOTED! !JAVA_PARAMS_QUOTED! "%SERVER_NAME%" --javadump !PARAMS_QUOTED! 2>> "%CLIENT_CMD_LOG%"
  set RC=%errorlevel%

  if %RC% == 0 del /Q "%CLIENT_CMD_LOG%" 1>nul 2>&1
goto:eof



@REM
@REM Utility functions
@REM

@REM
@REM Set environment variables for a non-server or nonexistent server command.
@REM
:installEnv
  call:readServerEnv "%WLP_INSTALL_DIR%\etc\server.env"
  call:installEnvDefaults
  call:serverEnvDefaults
goto:eof

@REM
@REM Set variable defaults for a non-server or nonexistent server command.
@REM
:installEnvDefaults
  call:readServerEnv "%WLP_INSTALL_DIR%\etc\default.env"

  if not defined WLP_DEFAULT_USER_DIR set WLP_DEFAULT_USER_DIR=!WLP_INSTALL_DIR!\usr
  if not defined WLP_USER_DIR set WLP_USER_DIR=!WLP_DEFAULT_USER_DIR!

  if not defined WLP_DEFAULT_OUTPUT_DIR set WLP_DEFAULT_OUTPUT_DIR=!WLP_USER_DIR!\servers
  if not defined WLP_OUTPUT_DIR set WLP_OUTPUT_DIR=!WLP_DEFAULT_OUTPUT_DIR!

  set SERVER_CONFIG_DIR=!WLP_USER_DIR!\servers\!SERVER_NAME!
goto:eof

@REM
@REM Set defaults for server variables.
@REM
:serverEnvDefaults
  set SERVER_OUTPUT_DIR=!WLP_OUTPUT_DIR!\!SERVER_NAME!

  if not defined LOG_DIR set LOG_DIR=!SERVER_OUTPUT_DIR!\logs
  if not defined LOG_FILE set LOG_FILE=console.log

  if NOT defined JAVA_HOME (
    if NOT defined JRE_HOME (
      if NOT defined WLP_DEFAULT_JAVA_HOME (
        @REM Use whatever java is on the path
        set JAVA_CMD_QUOTED="java"
      ) else (
        set JAVA_CMD_QUOTED="%WLP_DEFAULT_JAVA_HOME%\bin\java"
      )
    ) else (
      set JAVA_CMD_QUOTED="%JRE_HOME%\bin\java"
    )
  ) else (
    set JAVA_CMD_QUOTED="%JAVA_HOME%\bin\java"
  )
  
  if NOT defined IBM_JAVA_OPTIONS (
    set IBM_JAVA_OPTIONS=-Xshareclasses:name=liberty-%%u,nonfatal,cacheDir="%WLP_OUTPUT_DIR%\.classCache" -XX:ShareClassesEnableBCI -Xscmx60m -Xscmaxaot4m
  )
goto:eof

@REM
@REM Set environment variables for an existing server.
@REM
:serverEnv
  call:readServerEnv "%WLP_INSTALL_DIR%\etc\server.env"
  call:installEnvDefaults

  call:readServerEnv "%SERVER_CONFIG_DIR%\server.env"
  call:serverEnvDefaults
goto:eof

@REM
@REM Set environment variables and JVM_OPTIONS for an existing server.
@REM
:serverEnvAndJVMOptions
  call:serverEnv

  @REM By default, set -XX:MaxPermSize.  This option should be ignored by JVMs
  @REM that don't support the option.
  set JVM_OPTIONS=-XX:MaxPermSize=256m

  if exist "%SERVER_CONFIG_DIR%\jvm.options" (
    call:readJVMOptions "%SERVER_CONFIG_DIR%\jvm.options"
  ) else if exist "%WLP_INSTALL_DIR%\etc\jvm.options" (
    call:readJVMOptions "%WLP_INSTALL_DIR%\etc\jvm.options"
  )
goto:eof

@REM
@REM Read and set variables from the quoted file %1.  Empty lines and lines
@REM beginning with the hash character ('#') are ignored.  All other lines must
@REM be be of the form: VAR=VALUE
@REM
:readServerEnv
  if not exist %1 goto:eof
  for /f "usebackq eol=# delims== tokens=1,*" %%i in (%1) do set %%i=%%j
goto:eof

@REM
@REM Read the contents of the quoted file %1 and append the contents to
@REM %JVM_OPTIONS%.  Empty lines and lines beginning with the hash character
@REM ('#') are ignored.  All other lines are concatenated.
@REM
:readJVMOptions
  for /f "usebackq eol=# tokens=*" %%i in (%1) do set JVM_OPTIONS=!JVM_OPTIONS! "%%i"
goto:eof

@REM
@REM Set the current working directory for an existing server.
@REM
:serverWorkingDirectory
  if not exist "%SERVER_OUTPUT_DIR%" mkdir "%SERVER_OUTPUT_DIR%"
  cd /d "%SERVER_OUTPUT_DIR%"
goto:eof

@REM
@REM Set CLIENT_CMD_LOG.  Must only be called if the server exists.
@REM
:clientCmdLog
  if not exist "%LOG_DIR%" mkdir "%LOG_DIR%"
  set CLIENT_CMD_LOG=!LOG_DIR!\%1.log
goto:eof

@REM
@REM serverRunning: Return 0 if the server is running (.sLock file is in use), 
@REM                1 if not (file is not in use),
@REM
:serverRunning
  set SERVER_LOCK_FILE=!SERVER_OUTPUT_DIR!\workarea\.sLock
  if NOT EXIST "%SERVER_LOCK_FILE%" (
    set RC=1
  ) else (
    @REM If the server has locked the .sLock file, then type will output "The
    @REM process cannot access the file because another process has locked a
    @REM portion of the file."  Otherwise, the file is empty, and type will
    @REM output nothing, so we test for non-empty to check the lock.
    type "%SERVER_LOCK_FILE%" > "%SERVER_LOCK_FILE%.tmp" 2>&1
    set RC=1
    for %%i in ("%SERVER_LOCK_FILE%.tmp") do if %%~zi neq 0 set RC=0
    del "%SERVER_LOCK_FILE%.tmp"
  )
goto:eof

@REM
@REM serverExists: Return 0 if %SERVER_CONFIG_DIR% exists, or is "defaultServer" 
@REM                2 if server does not exist
@REM
:serverExists
  if "%SERVER_NAME%" == "defaultServer" (
    set RC=0
  ) else if NOT EXIST "%SERVER_CONFIG_DIR%" (
    if "%1" == "true" (
      !JAVA_CMD_QUOTED! !JAVA_PARAMS_QUOTED! --message:info.serverNotExist "%SERVER_NAME%"
    )
    set RC=2
  ) else (
    set RC=0
  )
goto:eof

@REM find the length of a string, ignoring quotes
@REM arg1 - variable name to store the length
@REM arg2 - the string whose length is to be found
:strLen
  SETLOCAL ENABLEDELAYEDEXPANSION
    set /A len = 0
    if .%2==. goto:strLen.return
    set "str=%~2"
    @REM This proved quite slow when removing one char at a 
    @REM time, so we now use a binary chop to get the length.
    set chop=64
    @REM Use tail-recursion to chomp through the string,
    @REM recording how much we have eaten as we go.
    :strLen.loop
      if defined str (
        set "tail=!str:~%chop%!"
        @REM If there is nothing left and the chop size was 2 or more
        @REM we might have overshot the end of the string, 
        @REM so halve the chop size and retry.
        if not defined tail if %chop% GEQ 2 (set /A chop /= 2 & goto strLen.Loop)
        @REM Otherwise record how much we removed and then process the tail.
        set /A len += chop
        set "str=!tail!"
        goto strLen.loop
      )
    @REM There is no string left to bite into, so set the named variable 
    @REM equal to the computed length.
  ENDLOCAL & SET "%~1=%len%"
goto:eof

@REM find the length of a variable
@REM arg1 - the name of the variable in which to store the length
@REM arg2 - the name of the variable to measure
:varLen
  SETLOCAL ENABLEDELAYEDEXPANSION
    set var=!%2!
    @REM replace double quotes with single quotes
    @REM and then surround the whole lot with double quotes
    set var="%var:"='%"
  ENDLOCAL & call :strLen %1 %var%
goto:eof

@REM Ensure double quotes are closed
@REM  arg1 - the name of the variable in which to ensure the double quotes are closed
:closeDoubleQuotes
  SETLOCAL ENABLEDELAYEDEXPANSION
    set string=!%1!
    @REM Count the number of double quotes in the string.
    @REM (To measure the length safely, we need a simple double-quoted string.)
    call:strLen lenWithQuotes "%string:"='%"
    call:strLen lenWithoutQuotes "%string:"=%"
    set /A numQuotes = lenWithQuotes - lenWithoutQuotes

    @REM Add a closing double quote if needed.
    set /A oddQuotes = numQuotes %% 2
    IF %oddQuotes%==1 SET string=%string%"
  ENDLOCAL & SET %1=%string%
goto:eof

@REM trim leading spaces from named variable
@REM   arg1 - the variable on which to operate
:trimLeadingSpaces
  SETLOCAL ENABLEDELAYEDEXPANSION
    set string=!%1!
    :trimLeadingSpaces.loop
      if not defined string goto:trimLeadingSpaces.return
      if "%string:~0,1%-%string:~0,1%" NEQ " - " (goto:trimLeadingSpaces.return) else set string=%string:~1%
      goto:trimLeadingSpaces.loop
    :trimLeadingSpaces.return
  ENDLOCAL & set %1=%string%
goto:eof

@REM Remove first argument from a list of arguments.
@REM   arg1   - the name of the environment variable on which to operate
@REM   arg... - (optional) the initial string value to use
@REM            If not supplied, operate on the contents of the named variable.
@REM NB: the string must not have unclosed double quotes!
:shiftVar
  if not defined %1 goto:eof
  SETLOCAL ENABLEDELAYEDEXPANSION
    set arg2=%2
    if not defined arg2 ENDLOCAL & goto shiftVar.recurse
    set string=%*
    set arg1=%1
    set arg2=%2
    call:varLen len1 arg1
    call:varLen len2 arg2
    @REM remove arg1
    set string=!string:~%len1%!
    call:trimLeadingSpaces string
    set string=!string:~%len2%!
    call:trimLeadingSpaces string
  ENDLOCAL & SET %1=%string%
goto:eof
:shiftVar.recurse
  @REM Use the secondary expansion afforded by CALL to expand the variable
  @REM named in arg1 as subsequent args on a recursive invocation.
  @REM To avoid infinite recursion, we first eliminate any initial spaces.
  @REM This may lead to an empty string, which will just be returned early.
  call:trimLeadingSpaces %1
  call:shiftVar %1 %%%1%%
goto:eof