
/* JavaScript content from js/com/ibm/mobile/appcenter/InstallerNative.js in folder common */
/*
 *  Licensed Materials - Property of IBM
 *  5725-G92 (C) Copyright IBM Corp. 2011, 2013. All Rights Reserved.
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */
 
define(["dojo/_base/lang"], function(lang){
    lang.getObject("com.ibm.mobile.appcenter.InstallerNative", true);
    
    var INIT_COMPLETE                  = "INIT_COMPLETE";
    var APPLICATION_LIST_UPDATE        = "APPLICATION_LIST_UPDATE";
    var ICON_LIST_UPDATE               = "ICON_LIST_UPDATE";
    var APPLICATION_CANCEL_UPDATE      = "APPLICATION_CANCEL_UPDATE";
    var APPLICATION_PROGRESSS_UPDATE   = "APPLICATION_PROGRESSS_UPDATE";
    var APPLICATION_FEEDBACK_REQUEST   = "FEEDBACK_REQUEST";
    
    function _alertError(msg) {
        /** @ignore */
        console.log("Error occured: " + msg);
        navigator.notification.alert(
                msg,  // message
                new function() {},      // no callback
                'Error',                // title
                null                    // buttonName, use defaults
            );
    }

    function _printList(pList)
    {
        var outString = "[";
        
        if(pList) {
            for(var i = 0 ; i < pList.length ; i++) {
                var entryStr = "{";
          
                if(pList[i] && pList[i].json) {
                    var entry = JSON.parse(pList[i].json);
                    entryStr += ("package: " + entry.pkg);
                    entryStr += (", version: " + entry.version);
                    entryStr += (", task_uid: " + (entry.task_uid ? entry.task_uid  : ""));
                }
                
                entryStr += "}";
            
                outString += entryStr;
            }
        }
        outString += "]";
        
        return outString;
    }

    com.ibm.mobile.appcenter.InstallerNative = {

        available: false,
            
        /**
         * The secure unique device ID (if registered, else undefined/null)
         * @type string
         */
        _deviceId: "",
        
        /**
         * The list of applications installed on this device
         * @type array
         */
        _applicationList: [],
        
        /**
         * The list of cached icons 
         * @type array
         */
        _iconList: [],
        
        
        /**
         * The list of applications in progress of some operation (download, remove)
         * @type array
         */
        _inprogressList: [],
        
        // TO BE REMOVED  
        _dpi: -1,
        _width: "",       
        _height: "",       
        _filter: "",
        
        nlsBundle: null,
        
        init: function(pNlsbundle,
        			   initCompleteCallback, 
                       iconUpdateCallback, 
                       applicationsUpdateCallback,
                       inprogressUpdateCallback,
                       cancelUpdateCallback,
                       feedbackRequestCallback,
                       deviceID) {

            console.log("com.ibm.mobile.appcenter.InstallerNative.init - [deviceID =]" + deviceID);

            this.nlsBundle = pNlsbundle;
            
            var me = this;
            this._init(deviceID,         
                       function(info) { 
                           console.log("com.ibm.mobile.appcenter.InstallerNative.init callback action: " + info.action);
                           if (info.action  == INIT_COMPLETE) {
                               me.available = true;
                               me._deviceId            = info.deviceId;
                               me._applicationList     = info.applicationList;
                               me._iconList            = info.iconList;
                               me._inprogressList      = info.inprogressList;
                               me._width               = info.width;        
                               me._height              = info.height;   
                               me._dpi				   = info.dpi;
                               me._filter              = info.filter;
                               console.log("initialized installer information:[deviceId, applicationList, iconList]=[" + 
                                           me._deviceId + "," +
                                           ", installed:" + _printList(me._applicationList) +                   
                                           ", inprogress:" + _printList(me._inprogressList) +
                                           "," + me._iconList + "]");
                               if (typeof initCompleteCallback === "function") {
                                   initCompleteCallback(info);
                               }   
                           } else if (info.action == APPLICATION_LIST_UPDATE && 
                                      typeof applicationsUpdateCallback === "function") {
                               me._applicationList     = info.applicationList;
                               me._inprogressList      = info.inprogressList;
                               console.log("applicationsUpdateCallback:[applicationList, _inprogressList]=[" + 
                                       _printList(me._applicationList) + "," + _printList(me._inprogressList) + "]");
                               applicationsUpdateCallback(me._applicationList, me._inprogressList);
                           } else if (info.action == ICON_LIST_UPDATE  && 
                                      typeof iconUpdateCallback === "function") {
                               me._iconList =  info.iconList;
                               console.log("iconUpdateCallback:[iconList]=[" + me._iconList + "]");
                               iconUpdateCallback(info.iconList);
                           }   else if (info.action == APPLICATION_CANCEL_UPDATE  && 
                                        typeof cancelUpdateCallback === "function") {
                               console.log("cancelUpdateCallback:[msg]=[" + info.msg + "]");
                               cancelUpdateCallback(info.msg);
                           }   else if (info.action == APPLICATION_PROGRESSS_UPDATE   && 
                                        typeof inprogressUpdateCallback === "function") {
                               console.log("inprogressUpdateCallback:[taskuid, msg]=[" +
                                           info.taskUid + ","  + info.msg +  "]");
                               inprogressUpdateCallback(info.taskUid, info.msg);
                           }   else if (info.action == APPLICATION_FEEDBACK_REQUEST   && 
                                        typeof feedbackRequestCallback === "function") {
                               console.log("feedbackRequestCallback:[pkg, version, stars, comment]=[" +
                                           info.pkg + ","  + info.version + info.stars + ","  + info.comment + "]");
                               // Assure we are not going to block the native thread!
                               setTimeout(function () {
                                   feedbackRequestCallback(info.pkg, info.version, info.stars, info.comment);
                               }, 0);                        
                           }       
                       },
                       function(e) {
                           console.log("Error initializing Installer: " + e);
                           _alertError( me.nlsBundle.initInstallerError.replace("%1",e) );  
                       }); 
        },            
        
        /**
         * Updated the saved information with new one
         * @ignore 
         */
        _init: function(pDeviceID, successCallback, failureCallback) {
            
            pDeviceID = pDeviceID ? pDeviceID : device.uuid;
            console.log("com.ibm.mobile.InstallerUtils.prototype.init [deviceID = ]" + pDeviceID);
            
            cordova.exec(successCallback,                   // Success callback from the plugin
                         failureCallback,                   // Error callback from the plugin
                         'com.ibm.mobile.InstallerPlugin',  // Tell cordova to run "com.ibm.mobile.InstallerPlugin" Plugin
                         'initInfo',                        // Tell plugin, which action we want to perform
                         [pDeviceID]);                      // Passing no arguments
            
            return true;
        },

        /**
         * Get the unique secure ID of this device
         * @ignore 
         */
        getDeviceId: function() {
            console.log("com.ibm.mobile.InstallerUtils.prototype.getDeviceId");

            return this._deviceId;
        },


        /**
         * Get the list of applications installed on this device
         * @ignore 
         */
        getInstalledApplications: function() {
            console.log("com.ibm.mobile.InstallerUtils.prototype.getInstalledApplications");

            return this._applicationList;
        },


        /**
         * Get the list of cached icons 
         * @ignore 
         */
        getIconsCache: function() {
            console.log("com.ibm.mobile.InstallerUtils.prototype.getIconsCache");

            return this._iconList;
        },

        /**
         * Get the list of in progress applications
         * @ignore 
         */
        getInprogressApplications: function() {
            console.log("com.ibm.mobile.InstallerUtils.prototype.getInprogressApplications");

            return this._inprogressList;
        },

        /**
         * Download and Install application
         * @ignore 
         */
        installApplication: function(pApp, successCallback, failureCallback) {
            console.log("com.ibm.mobile.InstallerUtils.prototype.installApplication");
                
            if(!pApp || !pApp.task_uid || !pApp.pkg || !pApp.version) {
                console.log("com.ibm.mobile.InstallerUtils.prototype.installApplication - failed inputs");
                return false;
            }
            var me = this;
            cordova.exec(successCallback,                   // Success callback from the plugin
            			 failureCallback,   				// Error callback
                         'com.ibm.mobile.InstallerPlugin', // Tell cordova to run "com.ibm.mobile.ApplicationManagementPlugin" Plugin
                         'installApplication',             // Tell plugin, which action we want to perform
                         [JSON.stringify(pApp)]);          // Passing list of args to the plugin

            return true;                                                                        
        },

        /**
         * Uninstall an application
         * @ignore 
         */
        removeApplication: function(pApp) {
            console.log("com.ibm.mobile.InstallerUtils.prototype.removeApplication");
            
            if(!pApp || !pApp.task_uid || !pApp.pkg || !pApp.version) {
                console.log("com.ibm.mobile.InstallerUtils.prototype.removeApplication - failed inputs ");
                return false;
            }
            var me = this;
            cordova.exec(null,                              // Success callback from the plugin
                         function(e) {                      // Error callback            	
                             _alertError( me.nlsBundle.removeApplicationError.replace("%1",e) );
                         },      
                         'com.ibm.mobile.InstallerPlugin',  // Tell cordova to run "com.ibm.mobile.ApplicationManagementPlugin" Plugin
                         'removeApplication',               // Tell plugin, which action we want to perform
                         [JSON.stringify(pApp)]);                                                          
            return true;                                                                        
        },

        /**
         * Cancel operation in progress for an application
         * @ignore 
         */
        cancelApplication: function(pApp) {
            console.log("com.ibm.mobile.InstallerUtils.prototype.cancelApplication");
            
            if(!pApp || !pApp.task_uid) {
                console.log("com.ibm.mobile.InstallerUtils.prototype.cancelApplication - failed inputs");
                return false;
            }
            var me = this;
            cordova.exec(null,                              // Success callback from the plugin
                         function(e) {                      // Error callback 
                             _alertError( me.nlsBundle.cancelApplicationError.replace("%1",e) );
                         },     
                         'com.ibm.mobile.InstallerPlugin',  // Tell cordova to run "com.ibm.mobile.ApplicationManagementPlugin" Plugin
                         'cancelApplication',               // Tell plugin, which action we want to perform
                         [JSON.stringify(pApp)]);                                                          
            return true;
        },

        /**
         * Cancel operation in progress for an application
         * @ignore 
         */
        downloadIcon: function(pURL) {
            console.log("com.ibm.mobile.InstallerUtils.prototype.downloadIcon from " + pURL);
            
            if(!pURL) {
                console.log("com.ibm.mobile.InstallerUtils.prototype.downloadIcon from - failed inputs");
                return false;
            }
            var me = this;
            cordova.exec(null,                              // Success callback from the plugin
                         function(e) {                      // Error callback 
                             _alertError( me.nlsBundle.downloadIconError.replace("%1",e) );
                         },         
                         'com.ibm.mobile.InstallerPlugin',  // Tell cordova to run "com.ibm.mobile.ApplicationManagementPlugin" Plugin
                         'downloadIcon',                    // Tell plugin, which action we want to perform
                         [pURL]);                           // Passing list of args to the plugin                  

            return true;
        },

        /**
         * Cancel operation in progress for an application
         * @ignore 
         */
        updateConnection: function(pUsername, pPassword, pServerURL, pSuccessCallback, pFailureCallback) {
            console.log("com.ibm.mobile.InstallerUtils.prototype.updateConnection User [" + pUsername + "] Password [*******] Server [" + pServerURL + "]");
            
            if(!pServerURL || !pUsername || !pPassword) {
                console.log("com.ibm.mobile.InstallerUtils.prototype.updateConnection from - failed inputs");
                return false;
            }
            
            cordova.exec(pSuccessCallback,                      // Success callback from the plugin
            			 pFailureCallback,         
                         'com.ibm.mobile.InstallerPlugin',      // Tell cordova to run "com.ibm.mobile.ApplicationManagementPlugin" Plugin
                         'updateConnection',                    // Tell plugin, which action we want to perform
                         [pUsername, pPassword, pServerURL]);   // Passing list of args to the plugin                  

            return true;
        },

        /**
         * Update the list of installed applications according to the server's list
         * pApps: JSON array of JSON objects describing the list of installed applications from server. 
         * @ignore 
         */
        syncInstalledApplications: function(pApps) {
            console.log("com.ibm.mobile.InstallerUtils.prototype.syncInstalledApplications" + JSON.stringify(pApps));
            var me = this;        
            cordova.exec(null,                              // Success callback from the plugin
                         function(e) {                      // Error callback 
                             _alertError( me.nlsBundle.syncInstalledApplicationsError.replace("%1",e) );
                         },   
                         'com.ibm.mobile.InstallerPlugin',  // Tell cordova to run "com.ibm.mobile.ApplicationManagementPlugin" Plugin
                         'syncInstalledApplications',       // Tell plugin, which action we want to perform
                         [JSON.stringify(pApps)]);          // Passing list of args to the plugin

            return true;                                                                        
        },


        /**
         * TO BE REMOVED LATER
         * 
         */
        hasDoneButton: function() {
            return false;
        },

        getDpi: function() {
            return this._dpi;      
        },

        getWidth: function() {
            return this._width;      
        },       

        getHeight: function() {
            return this._height;
        },

        getFilter: function() {
            return this._filter;
        },

        exitInstaller: function() {
            navigator.app.exitApp();           
        }            
    };
    
    com.ibm.mobile.appcenter.InstallerNative.DPI = {};
    com.ibm.mobile.appcenter.InstallerNative.DPI.DENSITY_LOW 		= 0;
    com.ibm.mobile.appcenter.InstallerNative.DPI.DENSITY_MEDIUM  	= 1;
    com.ibm.mobile.appcenter.InstallerNative.DPI.DENSITY_HIGH 		= 2;
    com.ibm.mobile.appcenter.InstallerNative.DPI.DENSITY_XHIGH  	= 3;
    
    
    com.ibm.mobile.appcenter.InstallerNative.ERROR_CODE = {};
    com.ibm.mobile.appcenter.InstallerNative.ERROR_CODE.NON_MARKET_DIS  = "INSTALL_NON_MARKET_APPS_DISABLED";
    com.ibm.mobile.appcenter.InstallerNative.ERROR_CODE.CONFIG_FAIL     = -4;
    com.ibm.mobile.appcenter.InstallerNative.ERROR_CODE.FAIL      		= -3;
    com.ibm.mobile.appcenter.InstallerNative.ERROR_CODE.SECURITY_FAIL   = -2;
    com.ibm.mobile.appcenter.InstallerNative.ERROR_CODE.CONNECT_FAIL    = -1;
    com.ibm.mobile.appcenter.InstallerNative.ERROR_CODE.SERVER_FAIL 	= 0;
    
    navigator.installer = com.ibm.mobile.appcenter.InstallerNative;
    
    return com.ibm.mobile.appcenter.InstallerNative;
});