
/* JavaScript content from js/com/ibm/mobile/appcenter/Settings.js in folder common */
/*
 *  Licensed Materials - Property of IBM
 *  5725-G92 (C) Copyright IBM Corp. 2011, 2013. All Rights Reserved.
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */
 
define(["dojo/_base/lang", "dijit/registry", "dojo/_base/connect"], function(lang, registry, connect) {
  
    lang.getObject("com.ibm.mobile.appcenter.Settings", true);
    
    var SETTINGS_SUBMIT_ID      = 'settingsSubmitButton';              
    var SETTINGS_USER_ID_ID     = 'settingsUserid';
    var SETTINGS_PASSWORD_ID    = 'settingsPassword';
    var SETTINGS_SERVER_URL_ID  = 'settingsServerurl';
    var SETTINGS_SSL_ID  		= 'settingsSSL';
    //

    var parse_servername = /((?:[0-9.\-A-Za-z]+)(?::\d+)?)(?:\/([^?#]+))?/;
    
    var parse_url = /(?:(http|https):\/\/)?((?:[0-9.\-A-Za-z]+)(?::\d+)?)(?:\/([^?#]+))?/;
    
    function isValidURL(url) {
        return url.match(parse_servername);
    }
    
    function isEmpty(str) {
    
        return !(str && str.length > 0);
    }
            
    function validateParameters(pBundle) {
    
        var errArray = [];
        
        if(isEmpty(registry.byId(SETTINGS_USER_ID_ID).get("value"))) {
            errArray.push(pBundle.userIDEmptyError);
        }
        if(isEmpty(registry.byId(SETTINGS_PASSWORD_ID).get("value"))) {
            errArray.push(pBundle.passowrdEmptyError);
        }
        if(isEmpty(registry.byId(SETTINGS_SERVER_URL_ID).get("value"))) {
            errArray.push(pBundle.serverUrlEmptyError);
        }
        else if(!isValidURL(registry.byId(SETTINGS_SERVER_URL_ID).get("value"))) {
            errArray.push(pBundle.invalidServerUrlError);
        }
    
        if(errArray.length > 0) {
           
        	navigator.notification.alert(errArray.join("\n"));
            return false;
        }
    
        return true;
    };
    
    function disableControls(toDisable) {
    
        registry.byId(SETTINGS_USER_ID_ID).set("disabled", toDisable);
        registry.byId(SETTINGS_PASSWORD_ID).set("disabled", toDisable);
        registry.byId(SETTINGS_SERVER_URL_ID).set("disabled", toDisable);
        registry.byId(SETTINGS_SUBMIT_ID).set("disabled", toDisable);
    };
                
    function showErrorMessage(errorMessage) {
    };

    com.ibm.mobile.appcenter.Settings = {

    	nlsBundle: null,
        user: '',
        password: '',
        url: '',
        contextRoot: '',
        onSubmit: null,
        WL_Client_ID: null,
        WL_Server_url:null,
        
        init: function(pUser, pPassword, pURL, pContextRoot, pOnSubmit, pNlsBundle) {
            
        	this.nlsBundle	= pNlsBundle;
            this.user       = pUser ? pUser : '';
            this.password   = pPassword ? pPassword : '';
            this.contextRoot = pContextRoot ? pContextRoot : '';
            this.url        = pURL ? pURL : '';
            this.onSubmit   = pOnSubmit;
            
            var fullUrl = '';
            if (this.url && this.contextRoot)
            	fullUrl = this.url + "/" + this.contextRoot;
            else if (this.url)
            	fullUrl = this.url;
            
            var result = parse_url.exec(fullUrl);
            
            
            registry.byId(SETTINGS_USER_ID_ID).set("value", this.user); 
            registry.byId(SETTINGS_PASSWORD_ID).set("value", this.password);
            
            if (result){
	            if (result[2] && result[3])
	            	registry.byId(SETTINGS_SERVER_URL_ID).set("value", result[2] + "/" + result[3]);
	            else
	            	registry.byId(SETTINGS_SERVER_URL_ID).set("value", result[2]);
            }
            registry.byId(SETTINGS_SSL_ID).set("value", result && result[1].toLowerCase() === "https" ? "on" : "off");
            
            var _this = this;
            connect.connect(registry.byId(SETTINGS_SUBMIT_ID), "onClick", function() {
                _this._submitCalled();
            });     
            
        },            
        
        onError: function(pMsg) {
            disableControls(false);
            showErrorMessage(pMsg);
        },
        
        onSuccess: function() {
            disableControls(false);
        },
        
        _submitCalled: function() {    
            document.getElementById('settingsBlurLink').focus();
            
            if(!validateParameters(this.nlsBundle))
                return;
        
            this.user       = registry.byId(SETTINGS_USER_ID_ID).get("value"); 
            this.password   = registry.byId(SETTINGS_PASSWORD_ID).get("value");
            var serverName        = registry.byId(SETTINGS_SERVER_URL_ID).get("value");
            var scheme = registry.byId(SETTINGS_SSL_ID).get("value") === "on" ? "https" : "http";
            
            var result = parse_url.exec(serverName);
            if (!result){
            	navigator.notification.alert(this.nlsBundle.invalidServerUrlError);
            	return;
            }
            
            var scheme_opt = result[1];
            var server = result[2];
            
            this.contextRoot = result[3];
            // Needs to remove the last characters of the context root if they are a "/" because not supported on WAS/LP
            this.contextRoot = this.contextRoot.replace(/\/+$/g,"");

            if (scheme_opt){
            	scheme = scheme_opt;
                registry.byId(SETTINGS_SSL_ID).set("value", result[1].toLowerCase() === "https" ? "on" : "off");
            }
            
            this.url = scheme +"://" + server;
            if(this.onSubmit) {
            	disableControls(true);
                this.onSubmit(this.user, this.password, this.url, this.contextRoot);
            }
        }
    };
    
    
    return com.ibm.mobile.appcenter.Settings;
});